package org.alltimeflashdreamer.mailwebform;

import org.alltimeflashdreamer.util.StringUtils;

import java.io.*;

import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;


/**
 * Title: AbstractMailWebFormServlet<br>
 * Description: The abstract superclass for MailWebForm's servlets collects
 * fields and methods used by each of the extending classes.
 *
 * @author Copyright (c) 2001-2004 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
abstract class AbstractMailWebFormServlet extends HttpServlet
	implements MailWebFormConstants
{
	/**
	 * Properties instance storing the values read from the global properties
	 * file
	 */
	GlobalProperties globalProperties = null;

	/**
	 * the list of known providers of a mailwebform hosted by this servlet,
	 * including the "system" provider
	 */
	MailFormProviderList mailFormProviders;

	/**
	 * Since this servlet makes no distinction between a GET and a POST, pass
	 * the GET requests over to the doPost method.
	 *
	 * @param request (see description of implementing doPost method)
	 * @param response (see description of implementing doPost method)
	 *
	 * @throws ServletException (see description of implementing doPost method)
	 * @throws IOException (see description of implementing doPost method)
	 */
	public void doGet(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException
	{
		doPost(request, response);
	}

	/**
	 * This method is declared only, to make sure that extending classes
	 * override, i.e. implement it.
	 *
	 * @param request (see description of implementing doPost method)
	 * @param response (see description of implementing doPost method)
	 *
	 * @throws ServletException (see description of implementing doPost method)
	 * @throws IOException (see description of implementing doPost method)
	 */
	public abstract void doPost(HttpServletRequest request,
		HttpServletResponse response) throws ServletException, IOException;

	/**
	 * Reads the settings from the global properties file which must reside in
	 * the application classes' root folder. In case this file isn't found,
	 * the upcoming exception is wrapped into a ServletException which is
	 * thrown, for convenience with each possible caller method.
	 *
	 * @throws MailWebFormException wrapped and commented IOException, if one was
	 * 		   thrown
	 */
	void readGlobalProperties() throws MailWebFormException
	{
		File propertyFile = null;

		try {
			//check whether it's necessary to update the global settings: (re)locate
			//the property file (in case it was moved or touched)
			propertyFile = getPropertyFile();

			//initialize or reload the global properties, if the property file has changed,
			//and (re)constructo the MailFormProviderList, if necessary
			boolean bMustInitializeMailFormProviders = true;

			if (globalProperties == null) {
				globalProperties = new GlobalProperties(propertyFile);
			} else {
				bMustInitializeMailFormProviders = globalProperties.checkReload(propertyFile);
			}

			if (bMustInitializeMailFormProviders) {
				mailFormProviders = new MailFormProviderList(globalProperties);
			}
		} catch (IOException ioe) {
			throw new MailWebFormException("Global properties file " +
				((propertyFile != null)
				? (propertyFile.getAbsolutePath() + propertyFile.getName() +
				" ") : "") + "not found or not readable!", ioe);
		}

		//tell the MailFormProviderList to check whether it's necessary to update 
		//settings of a or multiple providers
		mailFormProviders.checkProviderSettings();
	}

	/**
	 * Returns the abstract File instance pointing to the global properties
	 * file. If the file doesn't exist or the servlet runner user can't write
	 * to it, an exception is thrown.
	 *
	 * @return the global property file wrapper
	 *
	 * @throws MailWebFormException if the file is missing or not editable
	 */
	File getPropertyFile() throws MailWebFormException
	{
		File propertyFile = new File(getConfigDirProperty() + RESOURCENAME);

		if (!propertyFile.exists()) {
			throw new MailWebFormException("Global properties file " +
				((propertyFile != null)
				? (propertyFile.getAbsolutePath() + propertyFile.getName() +
				" ") : "") + "not found!");
		}

		if (!propertyFile.canWrite()) {
			throw new MailWebFormException("Global properties file " +
				((propertyFile != null)
				? (propertyFile.getAbsolutePath() + propertyFile.getName() +
				" ") : "") + "isn't writable for the servlet runner user!");
		}

		return propertyFile;
	}

	/**
	 * Returns the abstract File instance pointing to the user credentials
	 * file.
	 *
	 * @return the user credentials file wrapper
	 *
	 * @throws MailWebFormException if the system's configdir property isn't
	 * 		   set
	 */
	File getCredentialsFile() throws MailWebFormException
	{
		return new File(getConfigDirProperty() + PASSWORDFILE);
	}

	/**
	 * Reads the system property defining the app's config directory. If this
	 * property isn't set, an exception is thrown since the app can't work
	 * without this directory.
	 *
	 * @return the    path to the config dir, containing forward slashes only
	 * 		   and ending with such a forward slash
	 *
	 * @throws MailWebFormException if the property isn't defined
	 */
	static String getConfigDirProperty() throws MailWebFormException
	{
		String sProperty = System.getProperty(PROPERTY_CONFIGDIR);

		if (StringUtils.isEmpty(sProperty)) {
			throw new MailWebFormException("JVM property \"" +
				PROPERTY_CONFIGDIR +
				"\" isn't set!<br />Please refer to the documentation on how to do this.<br />" +
				"Without this property, the application isn't able to work.");
		}

		return StringUtils.appendFileSeparator(StringUtils.replaceBackslashes(
				sProperty), true);
	}

	/**
	 * Searches the list of parameter names case-insensitively for the given
	 * key and returns the first value, if found.
	 *
	 * @param request what to query for the URL parameters
	 * @param sKey the key that shall case-insensitively be searched for
	 *
	 * @return the first parameter value, if found, else null
	 */
	String getParameterIgnoreCase(HttpServletRequest request, String sKey)
	{
		String[] sParameterValuesIgnoreCase = getParameterValuesIgnoreCase(request,
				sKey);

		return ((sParameterValuesIgnoreCase == null) ? null
													 : sParameterValuesIgnoreCase[0]);
	}

	/**
	 * Searches the list of parameter names case-insensitively for the given
	 * key and returns the value list, if found.
	 *
	 * @param request what to query for the URL parameters
	 * @param sKey the key that shall case-insensitively be searched for
	 *
	 * @return the list parameter values, if found, else null
	 */
	String[] getParameterValuesIgnoreCase(HttpServletRequest request,
		String sKey)
	{
		Enumeration eParameterNames = request.getParameterNames();

		while (eParameterNames.hasMoreElements()) {
			String sNextKey = (String)eParameterNames.nextElement();

			if (sNextKey.equalsIgnoreCase(sKey)) {
				return request.getParameterValues(sNextKey);
			}
		}

		return null;
	}

	/**
	 * Prints the given text to the response's output writer, flushes and
	 * finally closes it.
	 *
	 * @param sText a text that is wrapped into a very simple HTML page
	 * @param response contains the client's url to write to
	 *
	 * @throws IOException thrown by the PrintWriter class, if any
	 */
	void printOutput(String sText, HttpServletResponse response)
		throws IOException
	{
		response.setContentType("text/html");

		PrintWriter out = response.getWriter();
		out.write("<html><head><title>" + APPNAME + "</title></head><body>");
		out.write(sText);
		out.write("</body></html>");
		out.flush();
		out.close();
	}

	/**
	 * Wraps the given error message into the JSPBean and redirects the client
	 * to the error page.
	 *
	 * @param mwfe the application-specific error object that is to be rendered
	 * @param request the client's request object
	 * @param response used for redirecting the client
	 *
	 * @throws IOException thrown by the PrintWriter class, if any
	 */
	void showErrorScreen(MailWebFormException mwfe, HttpServletRequest request,
		HttpServletResponse response) throws IOException
	{
		request.getSession().setAttribute(JSPBEAN, new JSPBean(mwfe));
		response.sendRedirect(SITE_ERROR);
	}

	/**
	 * Cleans up pointers and closes resources.
	 */
	public void destroy()
	{
		mailFormProviders = null;
		globalProperties = null;
	}
}
