package org.alltimeflashdreamer.mailwebform;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.alltimeflashdreamer.util.StringUtils;


/**
 * Title: AdminServlet<br>
 * Description: This is the controller servlet of the admin tool which allows
 * the administrator of the current MailWebForm installation to configure the
 * registered providers "on-line", i.e. using the browser. These settings
 * could be edited in the mailwebform.properties file manually, but because
 * the file resides on the serlvet runner's host machine and the application
 * is malfunctioning in case the file isn't edited properly, it's much easier
 * and safer to use this servlet.<br>
 * The admin tool has several different sites/views: The login screen, the
 * main window and the edit provider settings screen. The tasks of these sites
 * are:
 * 
 * <ul>
 * <li>
 * <i>Login Window:</i> In case it's the first time the administrator accesses
 * the admin tool, no login/password file of the administrator exists yet.
 * Therefore, the application accepts the first login as the reference and
 * stores the login and the password within the specified file (the password
 * is MD5 encoded). Each following access references the contents of the file.
 * If one wants to reset the login, he's got to delete the file. Because the
 * password file should be accessible for the administrator on the file system
 * only, it's assured that only the one who installed MailWebForm is able to
 * reset/change the admin account.
 * </li>
 * <li>
 * <i>Main Window:</i> The application's main window lists up the registered
 * providers (which contains at least of the built-in system account). On this
 * screen, the administrator may add a new, edit or remove an existing, change
 * the admin credentials or the password of an existing provider. Removing of
 * a provider is only allowed if it's not the system provider, because this
 * account contains the applications default/main settings which must be
 * defined (else the application doesn't work!). In case of removing a
 * provider, a JavaScript prompt asks you for comitting the deletion (enter a
 * "y" if you really want to remove a provider), but of course only if
 * JavaScript is activated and not filtered out of the stream by a firewall.
 * </li>
 * <li>
 * <i>Edit Provider Window:</i> In case the "add new provider" button was
 * clicked, the user must choose a previously unused provider name (selecting
 * an already defined one leads to an exception) and enter this provider's
 * settings, as for an existing one.<br>
 * In case the "edit selected provider" button was clicked, the user may edit
 * the settings of the selected provider. If the selected provider is a
 * "normal" one, these settings are "PROVIDERDIRECTORY" (which is the
 * directory of the provider's own overriding file and the mail template),
 * "TONAME", "TOADDRESS" (the name and the address to be used to send
 * MailWebForm's generated mails to, i.e. the creator of the mail sending
 * site), "SMTP_ALLOWOVERWRITE" (is the provider allowed to use an overriding
 * file, the default SMTP server settings entered by the administrator are
 * replaced with those settings) and "TEMPLATE" (name of the mail template to
 * be used). If the selected one is the provider named "system", the
 * application's default/global settings may be edited too, which are
 * "SMTP_HOST", "SMTP_PORT" (the default server to send the generated mails
 * to), "SUCCESS_URL", "ERROR_URL" (the sites to redirect the mail writer to
 * in case of successful or unsuccessful sending of the mail) and
 * "SPAMERROR_URL" (the site to redirect the client to in case he tried to
 * abuse MailWebForm, using illegal command line parameters), but neither an
 * overwrite path nor a template file may be defined, because the system
 * provider is not allowed to have such files.
 * </li>
 * <li>
 * <i>Edit Admin Credentials / Provider Password Window:</i> On this page, the
 * administrator may change the user name and the password of the admin
 * account itself or set a password for a specific provider. Since the
 * passwords are stored as a MD5-encoded string, this feature is absolutely
 * required.
 * </li>
 * <li>
 * <i>Edit Individual Provider Window:</i> In case the administrator set a
 * password for a certain provider, the application allows to log in using
 * this provider's name and the specified password. Having logged in using
 * this account, one can change the individual settings of a provider, i.e.
 * those that override the default ones defined by the administrator.
 * </li>
 * </ul>
 * 
 * Restrictions on the provider settings are as follows:
 * 
 * <ul>
 * <li>
 * The provider names must be valid Java identifiers, as described in the
 * java.lang.Character class, methods isJavaIdentifierStart and
 * isJavaIdentifierPart. To make it short: Don't use any "special" signs, i.e.
 * only characters and perhaps an underscore.
 * </li>
 * <li>
 * The provider names must be unique (case-insensitive check!).
 * </li>
 * <li>
 * A provider's own overriding file is named "&lt;somehow&gt;.properties" and
 * resides in the directory defined within the provider's settings (see
 * "PROVIDERDIRECTORY" description above) or in the application's dedicated
 * config directory, if this property isn't set. Therefore, this setting is
 * optional.
 * </li>
 * <li>
 * The "to name" setting is optional. If it isn't defined, the generated mail's
 * address line doesn't contain a recipient's name specification, i.e. the
 * address only.
 * </li>
 * <li>
 * The "provider is allowed to override" setting's hard-coded default is true
 * (stored as "YES"). If one wants to change this value to false, he must
 * unselect the corresponding checkbox.
 * </li>
 * <li>
 * The system provider's redirection URL's are optional. If such a site isn't
 * defined anywhere (neither for the system nor for the site provider),
 * MailWebForm prints it's own, unformatted, debug-like output.
 * </li>
 * <li>
 * The mail template setting is optional. If none is defined, MailWebForm
 * formats the mail by first adding the user-specific URL parameters, one at a
 * line, followed by a separator line and the mail text. If the setting is
 * defined, MailWebForm parses the template and replaces the placeholders with
 * the URL  parameters having the same name (case-insensitive check).
 * </li>
 * </ul>
 * 
 *
 * @author Copyright (c) 2001-2004 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
public class AdminServlet extends AbstractMailWebFormServlet
{
	private final static int MODE_SHOWLOGINSCREEN = 0;
	private final static int MODE_SHOWMAINSCREEN = 1;
	private final static int MODE_DOLOGIN = 2;
	private final static int MODE_DOLOGOUT = 3;
	private final static int MODE_ADDPROVIDER = 4;
	private final static int MODE_EDITPROVIDER = 5;
	private final static int MODE_REMOVEPROVIDER = 6;
	private final static int MODE_APPLYGLOBAL = 7;
	private final static int MODE_APPLYINDIVIDUAL = 8;
	private final static int MODE_STORESETTINGS = 9;
	private final static int MODE_CANCEL = 10;
	private final static int MODE_CHANGE_ADMIN_CREDENTIALS = 11;
	private final static int MODE_CHANGE_PROVIDER_PASSWORD = 12;

	/**
	 * This method can be called from different contexts, i.e. with different
	 * request parameters. The servlet then decides which mode is "active" and
	 * reacts accordingly. <br>
	 * The possible modes are:
	 * 
	 * <ul>
	 * <li>
	 * First call: Shows the login screen to the caller
	 * </li>
	 * <li>
	 * Main screen: The client sees the list of registered providers. On this
	 * screen, he's able to add, edit or remove providers.
	 * </li>
	 * <li>
	 * Provider settings screen: The user is able to edit a certain amount of
	 * settings of the currently selected provider. This number is varying
	 * depending on whether it's a new, an existing or the system provider.
	 * </li>
	 * <li>
	 * Action call ("logical site"): This mode is called lots of times, i.e.
	 * each time the administrator executes an action, such as logging in,
	 * changing the view, storing added or edited data or remove a provider.
	 * </li>
	 * <li>
	 * Individual provider screen: If the user logs in as a provider directly,
	 * he's able to edit the individual settings of a provider, i.e. those
	 * that override the global/default ones.
	 * </li>
	 * <li>
	 * Change admin credentials / provider password: The user may change the
	 * login name and/or the password of the admin account itself or the
	 * password of the previously selected provider.
	 * </li>
	 * </ul>
	 * 
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 *
	 * @throws ServletException in case problems occurred within the servlet
	 * 		   container
	 * @throws IOException in case problems occurred accessing a properties
	 * 		   file
	 */
	public void doPost(HttpServletRequest request, HttpServletResponse response)
		throws ServletException, IOException
	{
		int iMode = getMode(request);

		try {
			readGlobalProperties();

			switch (iMode) {
				case MODE_SHOWLOGINSCREEN:
					showLoginScreen(request, response);

					break;

				case MODE_SHOWMAINSCREEN:
				case MODE_CANCEL:
					showMainScreen(request, response, false);

					break;

				case MODE_DOLOGIN:
					doLogin(request, response);

					break;

				case MODE_DOLOGOUT:
					request.getSession().removeAttribute(CURRENTUSER);
					request.getSession().invalidate();
					showLoginScreen(request, response);

					break;

				case MODE_ADDPROVIDER:
					//check whether admin is (still) logged in
					if (!isUserLoggedIn(request) || !isCurrentUserAdmin(request)) {
						showLoginScreen(request, response);
					} else {
						request.getSession().setAttribute(JSPBEAN,
							new JSPBean("", globalProperties, mailFormProviders));
						response.sendRedirect(response.encodeRedirectURL(SITE_EDIT));
					}

					break;

				case MODE_EDITPROVIDER:
					editProviderSettings(request, response);

					break;

				case MODE_REMOVEPROVIDER:
					removeProvider(request, response);

					break;

				case MODE_APPLYGLOBAL:
					storeGlobalSettings(request, response, true);

					break;

				case MODE_APPLYINDIVIDUAL:
					storeIndividualSettings(request, response);

					break;

				case MODE_STORESETTINGS:
					storeGlobalSettings(request, response, false);

					break;

				case MODE_CHANGE_ADMIN_CREDENTIALS:
					changeCredentials(request, response, true);

					break;

				case MODE_CHANGE_PROVIDER_PASSWORD:
					changeCredentials(request, response, false);

					break;
			}
		} catch (MailWebFormException mwfe) {
			showErrorScreen(mwfe, request, response);
		}
	}

	/**
	 * Reads the parameters of the request and returns the appropriate constant
	 * (defined in this class). As the default, MODE_SHOWLOGINSCREEN is
	 * returned
	 *
	 * @param request contains the request parameters we need to have
	 *
	 * @return the internal run mode identifier
	 */
	private int getMode(HttpServletRequest request)
	{
		int iMode = MODE_SHOWLOGINSCREEN; //default is login screen

		try {
			//Important: "store" and "cancel" must be the first ones!
			if (request.getParameter(PARAM_LOGIN) != null) {
				iMode = MODE_DOLOGIN;
			} else if (request.getParameter(PARAM_STORE) != null) {
				iMode = MODE_STORESETTINGS;
			} else if (request.getParameter(PARAM_CANCEL) != null) {
				iMode = MODE_CANCEL;
			} else if (request.getParameter(PARAM_LOGOUT) != null) {
				iMode = MODE_DOLOGOUT;
			} else if (request.getParameter(PARAM_APPLYGLOBAL) != null) {
				iMode = MODE_APPLYGLOBAL;
			} else if (request.getParameter(PARAM_APPLYINDIVIDUAL) != null) {
				iMode = MODE_APPLYINDIVIDUAL;
			} else if (request.getParameter(PARAM_EDIT) != null) {
				iMode = MODE_EDITPROVIDER;
			} else if (request.getParameter(PARAM_REMOVE) != null) {
				iMode = MODE_REMOVEPROVIDER;
			} else if (request.getParameter(PARAM_PASSWORD_PROVIDER) != null) {
				iMode = MODE_CHANGE_PROVIDER_PASSWORD;
			} else if (request.getParameter(PARAM_CREDENTIALS_ADMIN) != null) {
				iMode = MODE_CHANGE_ADMIN_CREDENTIALS;
			} else if (request.getParameter(PARAM_ADD) != null) {
				iMode = MODE_ADDPROVIDER; //must be the very last one to be checked!
			}
		} catch (Exception ex) {}

		return iMode;
	}

	/**
	 * Shows the login screen, i.e. redirects the client to the login window
	 * after having (re)initialized the default JSPBean.
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 *
	 * @throws IOException problems on redirecting the client?
	 */
	private void showLoginScreen(HttpServletRequest request,
		HttpServletResponse response) throws IOException
	{
		request.getSession(true).setAttribute(JSPBEAN, new JSPBean());
		response.sendRedirect(response.encodeRedirectURL(SITE_LOGIN));
	}

	/**
	 * Searches for the file defined in <i>manually</i> in the application's
	 * global properties file. If this file is found and the URL parameters
	 * for the username and the password are equal to the contents of the
	 * file, the client is redirected to the application's main site, else an
	 * error site is shown.
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 *
	 * @throws MailWebFormException thrown in case the reading of the global
	 * 		   properties fails somehow
	 * @throws IOException problems on redirecting the client?
	 */
	private void doLogin(HttpServletRequest request,
		HttpServletResponse response) throws MailWebFormException, IOException
	{
		//check username and password input
		String sUserName = null;
		String sPassword = null;

		if (StringUtils.isEmpty(sUserName = request.getParameter(PARAM_USERNAME))) {
			throw new MailWebFormException("User name must not be empty!");
		}

		if (StringUtils.isEmpty(sPassword = request.getParameter(PARAM_PASSWORD))) {
			throw new MailWebFormException("Password must not be empty!");
		}

		//the system provider isn't allowed to log in!
		if (MailFormProvider.isSystemProvider(sUserName)) {
			throw new MailWebFormException(
				"The system provider isn't allowed to log in!<br />" +
				"To edit the global settings, please log in as the administrator and edit those " +
				"settings.");
		}

		//try reading the admin properties/credentials file
		Properties adminProps = new Properties();
		boolean bIsNewAccount = loadAdminCredentials(adminProps);

		//write new login and password to the file, if not yet stored, else
		//compare the relevant file's contents with the input values
		if (bIsNewAccount) {
			storeAdminCredentials(sUserName, sPassword);
			//don't use the admin user name but the word "admin"!
			sUserName = ADMIN;
		} else {
			//two possibilities: either it's the administrator or some provider
			String sAdminUsername = (String)adminProps.get(ADMIN + "." +
					USERNAME);
			String sAdminPassword = (String)adminProps.get(ADMIN + "." +
					PASSWORD);

			if (!sAdminUsername.equals(sUserName)) {
				//check whether it's another registered provider
				if (mailFormProviders.hasProvider(sUserName)) {
					//if so, compare the passwords of this provider
					String sProviderPassword = mailFormProviders.getPassword(sUserName);

					if (StringUtils.isEmpty(sProviderPassword) ||
							!sProviderPassword.equals(StringUtils.hash(
									sPassword))) {
						throw new MailWebFormException(
							"The entered password isn't correct!<br>Please go back and log in again.");
					}
				} else {
					//if not, throw an exception
					throw new MailWebFormException(
						"Wrong username/password combination!");
				}
			} else if (!sAdminPassword.equals(StringUtils.hash(sPassword))) {
				throw new MailWebFormException(
					"The entered password isn't correct!<br>Please go back and log in again.");
			} else {
				//don't use the admin user name but the word "admin"!
				sUserName = ADMIN;
			}
		}

		//store the name of the current verified user or the admin in the session
		request.getSession().setAttribute(CURRENTUSER, sUserName);

		//show the follower screen (admin: main, provider: individual)
		if (isCurrentUserAdmin(request)) {
			showMainScreen(request, response, bIsNewAccount);
		} else {
			showProviderScreen(request, response);
		}
	}

	/**
	 * Loads the user credentials of the admin user into the given
	 * java.util.Properties instance. Any possible exception is handled in
	 * here and wrapped into a MailWebFormException.
	 *
	 * @param adminProps where to fill the credentials in
	 *
	 * @return whether the admin account is a new one or not (occurs on the
	 * 		   very first access to this application or if the admin
	 * 		   credentials file has been (re)moved
	 *
	 * @throws MailWebFormException if the credentials file can't be read or
	 * 		   the servlet runner user isn't allowed to access this file or if
	 * 		   the config dir property hasn't been defined yet
	 */
	private boolean loadAdminCredentials(Properties adminProps)
		throws MailWebFormException
	{
		boolean bIsNewAccount = false;
		FileInputStream fis = null;

		try {
			fis = new FileInputStream(getCredentialsFile());
			adminProps.load(fis);
		} catch (FileNotFoundException fnfe) {
			bIsNewAccount = true;
		} catch (IOException ioe) {
			throw new MailWebFormException(
				"Caught error while trying to load the " +
				"credentials of the administrator!", ioe);
		} catch (SecurityException se) {
			throw new MailWebFormException(
				"The admin password file defined in the " + RESOURCENAME +
				" isn't accessible for this servlet.<br>Please make sure that the access " +
				"rights of the file or the servlet runner are appropriately set!");
		} finally {
			if (fis != null) {
				try {
					fis.close();
				} catch (Exception e) {}
			}
		}

		return bIsNewAccount;
	}

	/**
	 * Stores the given user name and the encoded password into a
	 * java.util.Properties instance and writes the admin credentials file using the
	 * Properties' store method. Any exception handling is done here, i.e. by
	 * transforming errors into a MailWebFormException.
	 *
	 * @param sUserName the user name of the admin account
	 * @param sPassword the (plain text) password of the admin account
	 *
	 * @throws MailWebFormException any problems while writing this file?
	 */
	private void storeAdminCredentials(String sUserName,
		String sPassword) throws MailWebFormException
	{
		Properties adminProps = new Properties();
		adminProps.put(ADMIN + "." + USERNAME, sUserName);
		adminProps.put(ADMIN + "." + PASSWORD, StringUtils.hash(sPassword));

		try {
			FileOutputStream fos = new FileOutputStream(getCredentialsFile());
			adminProps.store(fos,
				"\n# Please do not edit this file! MailWebForm would " +
				"not work correctly anymore.\n#");
			fos.close();
		} catch (FileNotFoundException fnfe) {
			throw new MailWebFormException(
				"The admin password file defined in the " + RESOURCENAME +
				" file doesn't exists.<br>Please install MailWebForm correctly!");
		} catch (IOException ioe) {
			throw new MailWebFormException(
				"The contents of admin password file defined in the " +
				RESOURCENAME +
				" couldn't be written.<br>Please try creating your " +
				"account again, in a few seconds. If this message should reappear, " +
				"contact your system administrator.");
		}
	}

	/**
	 * Returns the value of the current-user-name session attribute or null, if
	 * not defined.
	 *
	 * @param request where to get the session from
	 *
	 * @return the name of the currently logged in user
	 *
	 * @see #doLogin(HttpServletRequest, HttpServletResponse)
	 */
	private String getCurrentUserName(HttpServletRequest request)
	{
		return (String)request.getSession().getAttribute(CURRENTUSER);
	}

	/**
	 * Returns the whether the current-user-name session attribute equals
	 * the ADMIN constant value, i.e. whether the administrator is
	 * logged in or not.
	 *
	 * @param request where to get the session from
	 *
	 * @return whether currently the admin is logged in
	 *
	 * @see #doLogin(HttpServletRequest, HttpServletResponse)
	 */
	private boolean isCurrentUserAdmin(HttpServletRequest request)
	{
		return getCurrentUserName(request).equalsIgnoreCase(ADMIN);
	}

	/**
	 * Redirects the client to the app's main window, after having
	 * (re)initialized the appropriately configured JSPBean.
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 * @param bIsNewAccount whether the admin account was newly created or not
	 * 		  (typically on the very first access to this servlet)
	 *
	 * @throws IOException problems on redirecting the client?
	 */
	private void showMainScreen(HttpServletRequest request,
		HttpServletResponse response, boolean bIsNewAccount)
		throws IOException
	{
		//check whether admin is (still) logged in
		if (!isUserLoggedIn(request) || !isCurrentUserAdmin(request)) {
			showLoginScreen(request, response);

			return;
		}

		request.getSession().setAttribute(JSPBEAN,
			new JSPBean(bIsNewAccount, mailFormProviders.getProviderNames()));
		response.sendRedirect(response.encodeRedirectURL(SITE_MAIN));
	}

	/**
	 * Redirects the client to the provider's individual settings screen, after
	 * having (re)initialized the appropriately configured JSPBean.
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 *
	 * @throws IOException problems on redirecting the client?
	 */
	private void showProviderScreen(HttpServletRequest request,
		HttpServletResponse response) throws IOException
	{
		//check whether user is (still) logged in (not admin!)
		if (!isUserLoggedIn(request) || isCurrentUserAdmin(request)) {
			showLoginScreen(request, response);

			return;
		}

		request.getSession().setAttribute(JSPBEAN,
			new JSPBean(getCurrentUserName(request), globalProperties,
				mailFormProviders));
		response.sendRedirect(response.encodeRedirectURL(SITE_PROVIDER));
	}

	/**
	 * Redirects the client to the edit page, allowing him to edit the settings
	 * of the specified provider.
	 *
	 * @param sProviderName the name of the provider who's settings shall be
	 * 		  edited
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 *
	 * @throws IOException problems on redirecting the client?
	 */
	private void showEditScreen(String sProviderName,
		HttpServletRequest request, HttpServletResponse response)
		throws IOException
	{
		//check whether admin is (still) logged in
		if (!isUserLoggedIn(request) || !isCurrentUserAdmin(request)) {
			showLoginScreen(request, response);

			return;
		}

		request.getSession().setAttribute(JSPBEAN,
			new JSPBean(sProviderName, globalProperties, mailFormProviders));
		response.sendRedirect(response.encodeRedirectURL(SITE_EDIT));
	}

	/**
	 * Shows the "edit settings" screen, depending on the settings of the
	 * current provider. If this is the system account, then the additional,
	 * global system (default) settings are listed too.
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 *
	 * @throws MailWebFormException thrown if the URL parameter defining the
	 * 		   client isn't given / contained in the request
	 * @throws IOException problems on redirecting the client?
	 */
	private void editProviderSettings(HttpServletRequest request,
		HttpServletResponse response) throws MailWebFormException, IOException
	{
		//check whether admin is (still) logged in
		if (!isUserLoggedIn(request) || !isCurrentUserAdmin(request)) {
			showLoginScreen(request, response);

			return;
		}

		//make sure settings are up to date
		this.readGlobalProperties();

		//find out which provider's setting shall be edited
		String sProviderName = request.getParameter(PARAM_PROVIDER);

		if ((sProviderName == null) || (sProviderName.length() == 0)) {
			throw new MailWebFormException(
				"No provider is currently selected!<br>Please go back, " +
				"select a provider and restart editing.");
		}

		//redirect the client to the edit page that configures its content itself
		showEditScreen(sProviderName, request, response);
	}

	/**
	 * Reads the URL parameters, updates the settings of the added or edited
	 * provider and fully rewrites the global properties file <i>or</i> the
	 * admin credentials file.<br>
	 * Note: Checks against entered hosts and urls are avoided, it's up to the
	 * admin to make sure they're correct. The entered SMTP port value has to
	 * be a parseable number and above 0.
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 * @param bIsApply tells whether the apply or the save (and return) button
	 * 		  was clicked (i.e. where to go to after the successful storing)
	 *
	 * @throws MailWebFormException thrown on errors detected by the
	 * 		   application
	 * @throws IOException in case problems occurred accessing a properties
	 * 		   file or on redirecting the client to somewhere
	 */
	private void storeGlobalSettings(HttpServletRequest request,
		HttpServletResponse response, boolean bIsApply)
		throws MailWebFormException, IOException
	{
		//check whether admin is (still) logged in
		if (!isUserLoggedIn(request) || !isCurrentUserAdmin(request)) {
			showLoginScreen(request, response);

			return;
		}

		//check whether provider name is a legal Java identifier string
		String sProviderName = request.getParameter(PARAM_PROVIDER);
		String sPassword = null;

		//verify that the provider name isn't already in use and isn't equal
		//to the app name in case a new provider is to be created
		if (!StringUtils.isEmpty(request.getParameter(PARAM_ADD))) {
			validateProviderName(sProviderName);
			
			if (mailFormProviders.hasProvider(sProviderName)) {
				throw new MailWebFormException("Provider name \"" +
					sProviderName +
					"\" is already in use (case-insensitive check)!<br />" +
					"Please go back and select another one.");
			} else if (sProviderName.equalsIgnoreCase(MAILWEBFORM)) {
				throw new MailWebFormException("A provider's name must not " +
					"equal the application's name (case-insensitive check)!<br />" +
					"Please go back and select another one.");
			} else if (sProviderName.equalsIgnoreCase(ADMIN)) {
				throw new MailWebFormException("A provider's name must not " +
					"equal be equal to \"" + ADMIN + "\" (case-insensitive check)!<br />" +
					"Please go back and select another one.");
			}
		}

		//if the user came from one of the two change credentials sites, read the
		//new password value, else read the global properties of the provider and
		//store them into the MailFormProviderList
		boolean bMustWriteGlobalPropertiesFile = true;

		if (!StringUtils.isEmpty(request.getParameter(PARAM_CHANGE_CREDENTIALS))) {
			sPassword = readPassword(request);

			//if the admin credentials have been changed, rewrite the admin credentials
			//file, else store the provider's new password
			if (sProviderName.equalsIgnoreCase(ADMIN)) {
				String sUserName = request.getParameter(PARAM_USERNAME);

				if (StringUtils.isEmpty(sUserName)) {
					throw new MailWebFormException(
						"The login name must not be an empty string!");
				}

				storeAdminCredentials(sUserName, sPassword);
				bMustWriteGlobalPropertiesFile = false;
			} else {
				mailFormProviders.updatePassword(sProviderName,
					StringUtils.hash(sPassword));
			}
		} else {
			//verify validity of the selected provider name against the
			//criteria defined in the java.lang.Character class
			validateProviderName(sProviderName);

			//update the "personal" settings of the last edited provider (if sPassword isn't
			//null or "", a new provider has been created -> store the password too)
			String sProviderDirectory = request.getParameter(PARAM_PROVIDERDIRECTORY);

			try {
				if (sProviderDirectory.startsWith(".")) {
					sProviderDirectory = getServletContext().getRealPath(sProviderDirectory);
				}
			} catch (Throwable t) {
				//ignore this (probably a MalformedURLException), since it means that 
				//it's not a relative path...
			}

			mailFormProviders.updateProvider(sProviderName, sProviderDirectory,
				request.getParameter(PARAM_TONAME),
				request.getParameter(PARAM_TOADDRESS),
				request.getParameter(PARAM_OVERWRITE),
				request.getParameter(PARAM_TEMPLATE), globalProperties);

			if (!StringUtils.isEmpty(sPassword)) {
				mailFormProviders.updatePassword(sProviderName,
					StringUtils.hash(sPassword));
			}

			//update the global parameters, in case they've been edited (incl. checks)
			if (MailFormProvider.isSystemProvider(sProviderName)) {
				globalProperties.setSMTPPort(readSMTPPort(request, false));

				//checks against hosts and urls are avoided, it's up to the admin to
				//make sure they're correct
				globalProperties.setSMTPHost(request.getParameter(
						PARAM_SMTPHOST));
				globalProperties.setSuccessURL(request.getParameter(
						PARAM_SUCCESSURL));
				globalProperties.setErrorURL(request.getParameter(
						PARAM_ERRORURL));
				globalProperties.setSpamErrorURL(request.getParameter(
						PARAM_SPAMURL));
			}
		}

		//write file using the latest data, if necessary; in case this produces an
		//error, return immediately (error site is shown there)
		if (bMustWriteGlobalPropertiesFile) {
			writeGlobalPropertiesFile();
		}

		//redirect user to appropriate site
		if (bIsApply) {
			editProviderSettings(request, response);
		} else {
			showMainScreen(request, response, false);
		}
	}
	
	/**
	 * Checks whether the given provider name is a valid one. If not,
	 * a describing exception is thrown.
	 * 
	 * @param sProviderName	whether the provider name starts with a
	 * java start identifier and contains of java identifiers
	 * @throws MailWebFormException if those needs aren't met
	 */
	private void validateProviderName(String sProviderName)
		throws MailWebFormException
	{
		if (StringUtils.isEmpty(sProviderName))
			throw new MailWebFormException("The provider name must not be an empty string!");
		
		try {
			char[] cProvider = sProviderName.toCharArray();

			for (int i = 0; i < cProvider.length; i++) {
				if ((i == 0) &&
						!Character.isJavaIdentifierStart(cProvider[0])) {
					throw new MailWebFormException(
						"The first character of the provider name is not a valid " +
						"identifying character!<br>Please go back and select another one!");
				} else if ((i != 0) &&
						!Character.isJavaIdentifierPart(cProvider[i])) {
					throw new MailWebFormException("Character #" + (i + 1) +
						" of the provider name is not " +
						"a valid identifying character!<br>Please go back and select another one!");
				}
			}
		} catch (Throwable t) {
			if (t instanceof MailWebFormException) {
				throw (MailWebFormException)t;
			}

			//in case some NullPointerException or ArrayIndexOutOfBoundsException
			//came up during getting or parsing the provider name
			throw new MailWebFormException("Missing or invalid provider name input!",
				t);
		}
	}

	/**
	 * Reads the SMTP port URL parameter and verifies it against being an int
	 * of value 0 or higher (or above 0, depending on the given boolean).
	 *
	 * @param request where to read the URL parameter from
	 * @param bIncludingZero whether to allow 0 or not
	 *
	 * @return the parsed and verified port number
	 *
	 * @throws MailWebFormException if the URL parameter isn't a number of one
	 * 		   that is below 0
	 */
	private int readSMTPPort(HttpServletRequest request, boolean bIncludingZero)
		throws MailWebFormException
	{
		int iPort = 0;

		try {
			iPort = new Integer(request.getParameter(PARAM_SMTPPORT)).intValue();

			if ((bIncludingZero && (iPort < 0)) ||
					(!bIncludingZero && (iPort <= 0))) {
				throw new MailWebFormException(
					"The entered SMTP port value is not a legal port!");
			}
		} catch (NumberFormatException nfe) {
			throw new MailWebFormException("The entered SMTP port value is not a number!",
				nfe);
		}

		return iPort;
	}

	/**
	 * Checks the password input (must not be empty and are two parameters!)
	 * and returns the first one of the two.
	 *
	 * @param request where to read the URL parameters from
	 *
	 * @return the validated password to be used/stored
	 *
	 * @throws MailWebFormException if the passwords are empty or not twice the
	 * 		   same
	 */
	private String readPassword(HttpServletRequest request)
		throws MailWebFormException
	{
		String[] sPasswords = sPasswords = request.getParameterValues(PARAM_PASSWORD);

		if ((sPasswords == null) || (sPasswords.length != 2) ||
				StringUtils.isEmpty(sPasswords[0]) ||
				StringUtils.isEmpty(sPasswords[1]) ||
				!sPasswords[0].equals(sPasswords[1])) {
			throw new MailWebFormException(
				"Passwords must not be empty and be twice the same!");
		}

		return sPasswords[0];
	}

	/**
	 * Stores the individual property file of the provider belonging to the
	 * currently logged in user.
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 *
	 * @throws MailWebFormException thrown on errors detected by the
	 * 		   application
	 * @throws IOException in case problems occurred accessing the individual
	 * 		   property file or on redirecting the client to somewhere
	 */
	private void storeIndividualSettings(HttpServletRequest request,
		HttpServletResponse response) throws MailWebFormException, IOException
	{
		//check whether user is (still) logged in (not admin!)
		if (!isUserLoggedIn(request) || isCurrentUserAdmin(request)) {
			showLoginScreen(request, response);

			return;
		}

		//update the current user's provider with the individual settings
		MailFormProvider provider = mailFormProviders.getProvider(getCurrentUserName(
					request));
		provider.updateIndividualSettings(request.getParameter(PARAM_TONAME),
			request.getParameter(PARAM_TOADDRESS),
			request.getParameter(PARAM_SMTPHOST), readSMTPPort(request, true),
			request.getParameter(PARAM_SUCCESSURL),
			request.getParameter(PARAM_ERRORURL),
			request.getParameter(PARAM_TEMPLATE));

		try {
			new IndividualPropertiesFileWriter().writeFile(provider);
		} catch (IOException ioe) {
			throw new MailWebFormException(
				"Error occurred during the writing of " +
				"the individual property file!", ioe);
		}

		showProviderScreen(request, response);
	}

	/**
	 * Removes the given provider from the list of providers and stores the
	 * modified global properties file. Finally, the user is redirected to the
	 * main window in case the updating of the property file was successful.
	 *
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 *
	 * @throws MailWebFormException thrown on errors detected by the
	 * 		   application
	 * @throws IOException in case problems occurred accessing a properties
	 * 		   file or on redirecting the client to somewhere
	 */
	private void removeProvider(HttpServletRequest request,
		HttpServletResponse response) throws MailWebFormException, IOException
	{
		//check whether admin is (still) logged in
		if (!isUserLoggedIn(request) || !isCurrentUserAdmin(request)) {
			showLoginScreen(request, response);

			return;
		}

		//verify it's not the system account
		String sProviderName = request.getParameter(PARAM_PROVIDER);

		if (MailFormProvider.isSystemProvider(sProviderName)) {
			throw new MailWebFormException(
				"It's not allowed to remove the system provider!");
		}

		//update list, ...
		readGlobalProperties();

		//... remove the provider ...
		mailFormProviders.removeProvider(sProviderName);

		//... and rewrite the settings, redirecting the user
		//to the main window (in case writing was successful)
		writeGlobalPropertiesFile();
		showMainScreen(request, response, false);
	}
	
	/**
	 * Changes the change credentials page of the admin or some provider,
	 * depending on the given boolean. The JSPBean is prepared appropriately.
	 * 
	 * @param request contains the request parameters we need to have
	 * @param response contains the client's url to write to
	 * @param bOfAdmin whether to go to the admin change page or not
	 *
	 * @throws MailWebFormException thrown on errors detected by the
	 * 		   application
	 * @throws IOException in case problems on redirecting the 
	 * client to somewhere
	 */
	private void changeCredentials(HttpServletRequest request,
			HttpServletResponse response, boolean bOfAdmin)
		throws MailWebFormException, IOException
	{
		//check whether admin is (still) logged in
		if (!isUserLoggedIn(request) || !isCurrentUserAdmin(request)) {
			showLoginScreen(request, response);

			return;
		}
		
		if (bOfAdmin) {
			Properties adminProps = new Properties();
			loadAdminCredentials(adminProps);
			request.getSession().setAttribute(JSPBEAN,
				new JSPBean(request.getParameter(PARAM_PROVIDER),
					(String)adminProps.get(ADMIN + "." + USERNAME)));
		} else {
			//credentials of system provider must not be changed
			String sProviderName = request.getParameter(PARAM_PROVIDER);
			if (sProviderName.equalsIgnoreCase(SYSTEMPROVIDER)) {
				throw new MailWebFormException("It's not allowed to enter a " +
					"password for the system provider!");
			}
			
			request.getSession().setAttribute(JSPBEAN,
					new JSPBean(sProviderName, (String)null));
		}

		response.sendRedirect(response.encodeRedirectURL(
				SITE_CHANGE_CREDENTIALS));
	}

	/**
	 * Returns whether the user's login cookie is set and not out of age.
	 *
	 * @param request contains the request parameters we need to have
	 *
	 * @return boolean whether the login is still valid or not
	 */
	private boolean isUserLoggedIn(HttpServletRequest request)
	{
		return !StringUtils.isEmpty(getCurrentUserName(request));
	}

	/**
	 * Forces the GlobalPropertiesFileWriter to write a new version of the
	 * file.
	 *
	 * @throws MailWebFormException thrown in case the writing of the property
	 * 		   file failed somehow
	 */
	private synchronized void writeGlobalPropertiesFile()
		throws MailWebFormException
	{
		try {
			new GlobalPropertiesFileWriter().writeFile(globalProperties,
				mailFormProviders, getPropertyFile());
		} catch (IOException ioe) {
			throw new MailWebFormException(
				"Error occurred during the writing of " + "the " +
				RESOURCENAME + " file!", ioe);
		}
	}
}
