package org.alltimeflashdreamer.mailwebform;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import java.util.Properties;


/**
 * Title: GlobalProperties<br>
 * Description: Reloadable implementation of the java.util.Properties class,
 * ready to return certain properties through specific accessor methods.
 *
 * @author Copyright (c) 2001-2004 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
public class GlobalProperties extends Properties implements MailWebFormConstants
{
	/**
	 * last-modified setting of the global properties file; is stored to know
	 * when the file must be reread because it's been changed
	 */
	private long lPropertyFileLastModified = 0L;

	/**
	 * Reads the contents of the specified property file into the internal list
	 * of properties.
	 *
	 * @param propertyFile the file containing the properties, as specified by
	 * 		  the java.util.Properties class
	 *
	 * @throws IOException if the reading of the file fails somehow
	 *
	 * @see #checkReload(java.io.File)
	 */
	GlobalProperties(File propertyFile) throws IOException
	{
		checkReload(propertyFile);
	}

	/**
	 * Checks whether the save date of the property file has changed or not. If
	 * so, the list is emptied, the values are re-read, and true (i.e. action
	 * done) is returned, else nothing is done and  false is returned.
	 *
	 * @param propertyFile the file containing the properties, as specified by
	 * 		  the java.util.Properties class
	 * @return if the property file was (successfully) reloaded
	 * @throws IOException if the reading of the file fails somehow
	 */
	synchronized boolean checkReload(File propertyFile)
		throws IOException
	{
		//compare last modification dates; return, if equal (i.e. unmodified since last load)
		long lSaveDate = propertyFile.lastModified();

		if (lSaveDate == lPropertyFileLastModified) {
			return false;
		}

		//read global properties here and pass the user-defined entries the MailFormProviderList
		lPropertyFileLastModified = lSaveDate;
		clear();

		FileInputStream fis = new FileInputStream(propertyFile);
		load(fis);

		try {
			fis.close();
		} catch (IOException ioe) {}

		return true;
	}

	/**
	 * Returns the global setting for the smtp server's host name.
	 *
	 * @return the name of the default SMTP host to be used
	 */
	public String getSMTPHost()
	{
		return getProperty(SMTP_HOST);
	}

	/**
	 * Adds or replaces the stored value for the SMTP host.
	 *
	 * @param sSMTPHost the (new) SMTP host name to be stored
	 */
	void setSMTPHost(String sSMTPHost)
	{
		setProperty(SMTP_HOST, sSMTPHost);
	}

	/**
	 * Returns the global setting for the smtp server's port value.
	 *
	 * @return the number of the default SMTP port to be used
	 */
	public int getSMTPPort()
	{
		try {
			return new Integer(getProperty(SMTP_PORT)).intValue();
		} catch (NumberFormatException nfe) {
			return 0;
		}
	}

	/**
	 * Adds or replaces the stored value for the SMTP port.
	 *
	 * @param iSMTPPort the (new) SMTP port number to be stored
	 */
	void setSMTPPort(int iSMTPPort)
	{
		setProperty(SMTP_PORT, new Integer(iSMTPPort).toString());
	}

	/**
	 * Returns the global setting for the site that is shown in case of a
	 * successful delivery of the generated mail; may be unset.
	 *
	 * @return the default success site to be shown or ""
	 */
	public String getSuccessURL()
	{
		return getProperty(SUCCESS_URL, ""); //may be undefined
	}

	/**
	 * Adds or replaces the stored value for the success URL.
	 *
	 * @param sSuccessURL the (new) success URL to be stored
	 */
	void setSuccessURL(String sSuccessURL)
	{
		setProperty(SUCCESS_URL, sSuccessURL);
	}

	/**
	 * Returns the global setting for the site that is shown in case of an
	 * unsuccessful delivery of the generated mail; may be unset.
	 *
	 * @return the default error site to be shown or ""
	 */
	public String getErrorURL()
	{
		return getProperty(ERROR_URL, ""); //may be undefined
	}

	/**
	 * Adds or replaces the stored value for the error URL.
	 *
	 * @param sErrorURL the (new) error URL to be stored
	 */
	void setErrorURL(String sErrorURL)
	{
		setProperty(ERROR_URL, sErrorURL);
	}

	/**
	 * Returns the global setting for the site that is shown in case of an
	 * illegal calling of the application (by hackers or spammers); may be
	 * unset.
	 *
	 * @return the default spam error site to be shown or ""
	 */
	public String getSpamErrorURL()
	{
		return getProperty(SPAMERROR_URL, ""); //may be undefined
	}

	/**
	 * Adds or replaces the stored value for the spam error URL.
	 *
	 * @param sSpamErrorURL the (new) spam error URL to be stored
	 */
	void setSpamErrorURL(String sSpamErrorURL)
	{
		setProperty(SPAMERROR_URL, sSpamErrorURL);
	}
}
