package org.alltimeflashdreamer.mailwebform;

import java.io.*;


/**
 * Title: GlobalPropertiesFileWriter<br>
 * Description: This class is designed to write the global properties file,
 * from scratch, to make sure that each additional text is kept
 * (java.util.Properties has its own, ugly format...). While the constructor
 * is inactive, the job is done within the writeFile method.
 *
 * @author Copyright (c) 2001-2004 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
class GlobalPropertiesFileWriter implements MailWebFormConstants
{
	private final static String LEAD = "#" + NEWLINE +
		"# This file contains the implementation specific properties." +
		NEWLINE +
		"# The parameters vary with each server and/or servlet runner!" +
		NEWLINE + "#" + NEWLINE +
		"# Each optional entry can be deactivated by adding a \"#\" at the beginning of its line." +
		NEWLINE + "#" + NEWLINE;
	private final static String D_SMTPH =
		"# ******************************************************************" +
		NEWLINE + "# Global default, but overrideable properties:" + NEWLINE +
		"# --------------------------------------------" + NEWLINE +
		"# Overwriting of SMTP host, SMTP port and to-address is allowed by default; this may be overwritten" +
		NEWLINE +
		"# for a specific provider in the \"Provider Setup\" section below adding the line" +
		NEWLINE + "# " + MAILFORMPROVIDER + ".<provider_name>." +
		SMTP_ALLOWOVERWRITE + "=NO" + NEWLINE + "# , if wished." + NEWLINE +
		"#" + NEWLINE +
		"# the default SMTP host to use; if overwriting is allowed, the provider may overwrite" +
		NEWLINE +
		"# it in his/her individual properties file located in the directory named <PROVIDERDIRECTORY>." +
		NEWLINE;
	private final static String D_SMTPP = "#" + NEWLINE +
		"# the default SMTP port to use; if overwriting is allowed, the provider may overwrite" +
		NEWLINE +
		"# it in his/her individual properties file located in the directory named <PROVIDERDIRECTORY>." +
		NEWLINE;
	private final static String D_SUCC = "#" + NEWLINE +
		"# optional: the default URL that a client shall be redirected to in case of successful sending of the email;" +
		NEWLINE +
		"# the provider may overwrite it in his/her individual properties file located in the directory named <PROVIDERDIRECTORY>;" +
		NEWLINE +
		"# if none is specified, the servlet's own (hard-coded) output is shown to the client." +
		NEWLINE;
	private final static String D_ERROR = "#" + NEWLINE +
		"# optional: the default URL that a client shall be redirected to in case an error occurred during sending of the email;" +
		NEWLINE +
		"# the provider may overwrite it in his/her individual properties file located in the directory named <PROVIDERDIRECTORY>;" +
		NEWLINE +
		"# if none is specified, the servlet's own (hard-coded) output is shown to the client. Only the hard-coded output" +
		NEWLINE + "# shows the error messages!" + NEWLINE;
	private final static String D_SPAM = "#" + NEWLINE +
		"# Global, not overrideable properties:" + NEWLINE +
		"# ------------------------------------" + NEWLINE +
		"# optional: the error site to be shown when a not registered provider tries to access the servlet" +
		NEWLINE + "# (probably somebody tried to spam...)" + NEWLINE;
	private final static String D_PROVLI =
		"# ******************************************************************" +
		NEWLINE + "# Provider Setup:" + NEWLINE + "# ---------------" +
		NEWLINE +
		"# Note: The settings in this section can be adjusted by the web tool easily!" +
		NEWLINE + "#" + NEWLINE +
		"# Because each implementing site is allowed to send the mails to a single address only," +
		NEWLINE +
		"# the properties for each such site are defined here, in the global properties file." +
		NEWLINE +
		"# The setup consists of two parts: The defining list providers and the list of the settings of" +
		NEWLINE + "# each defined provider." + NEWLINE + "#" + NEWLINE +
		"# Following the comma-separated list of registered mail form providers (attention: case sensitive!)." +
		NEWLINE + "# \"system\" is the only required entry." + NEWLINE;
	private final static String D_PROVIN =
		"# For each registered provider, the individual properties are defined here:" +
		NEWLINE +
		"# - the \"To Address\" (to prevent spamming, it's allowed to send to one address only)" +
		NEWLINE +
		"# - optional: files called <somename>.properties may exist in the directory specified in <PROVIDERDIRECTORY>." +
		NEWLINE +
		"#   If no such file exists, the default settings are taken." +
		NEWLINE +
		"# - optional: the name leading the email's to-address; this value is special because it may be overwritten in two different" +
		NEWLINE +
		"#   places: in the provider's properties file AND in a hidden input tag within the provider's html form." +
		NEWLINE +
		"#   The hierarchy of overwriting is as follows (top priority first): input tag, provider's settings, global settings" +
		NEWLINE + "#" + NEWLINE;

	/**
	 * (Re-)Writes the global properties file, using the given values (the
	 * default system settings are stored within the provider "system").
	 *
	 * @param globalProperties the list of properties containing the global
	 * 		  values
	 * @param mailFormProviders the list of registered providers, including the
	 * 		  system/default
	 * @param propertyFile the file to be written
	 *
	 * @throws IOException Description of the Exception
	 */
	void writeFile(GlobalProperties globalProperties,
		MailFormProviderList mailFormProviders, File propertyFile)
		throws IOException
	{
		//append global descriptions and settings
		StringBuffer sb = new StringBuffer();
		sb.append(LEAD).append(NEWLINE);
		sb.append(D_SMTPH);
		sb.append(SMTP_HOST).append('=').append(globalProperties.getSMTPHost())
			.append(NEWLINE);
		sb.append(D_SMTPP);
		sb.append(SMTP_PORT).append('=').append(globalProperties.getSMTPPort())
			.append(NEWLINE);
		sb.append(D_SUCC);
		sb.append(SUCCESS_URL).append('=')
			.append(globalProperties.getSuccessURL()).append(NEWLINE);
		sb.append(D_ERROR);
		sb.append(ERROR_URL).append('=').append(globalProperties.getErrorURL())
			.append(NEWLINE);
		sb.append(D_SPAM);
		sb.append(SPAMERROR_URL).append('=')
			.append(globalProperties.getSpamErrorURL()).append(NEWLINE).append(NEWLINE);

		//append list of registered providers and the provider-specific settings
		String[] sProviderNames = mailFormProviders.getProviderNames();
		String sProviderList = "";

		if (sProviderNames != null) {
			for (int i = 0; i < sProviderNames.length; i++) {
				sProviderList += (sProviderNames[i] +
					((i < (sProviderNames.length - 1)) ? "," : ""));
			}
		}

		sb.append(D_PROVLI).append(MAILFORMPROVIDERLIST).append('=')
			.append(sProviderList).append(NEWLINE).append(NEWLINE);

		if (sProviderNames != null) {
			for (int i = 0; i < sProviderNames.length; i++) {
				String[] sNextPart = mailFormProviders.getGlobalPropertiesPart(sProviderNames[i]);

				for (int j = 0; (sNextPart != null) && (j < sNextPart.length);
						j++) {
					sb.append(sNextPart[j]).append(NEWLINE);
				}

				sb.append(NEWLINE);
			}
		}

		//write file
		BufferedWriter bw = new BufferedWriter(new FileWriter(propertyFile));
		bw.write(sb.toString());
		bw.flush();
		bw.close();
	}
}
