package org.alltimeflashdreamer.mailwebform;

import org.alltimeflashdreamer.util.StringUtils;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;


/**
 * Title: IndividualPropertiesFileWriter<br>
 * Description: This class is designed to write the individual properties file
 * of a certain provider, from scratch, to make sure that each additional text
 * is kept (java.util.Properties has its own, ugly format...). While the
 * constructor is inactive, the job is done within the writeFile method.
 *
 * @author Copyright (c) 2001-2004 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
class IndividualPropertiesFileWriter implements MailWebFormConstants
{
	private final static String COMMENT_SMTPHOST = "# the value of the SMTP host to use";
	private final static String COMMENT_SMTPPORT = "# the value of the SMTP port to use (typically 25)";
	private final static String COMMENT_TONAME =
		"# the URL that a client shall be redirected to " +
		"in case of successful sending of the email";
	private final static String COMMENT_TOADDRESS =
		"# the URL that a client shall be redirected to " +
		"in case an error occurred during sending of the email";
	private final static String COMMENT_SUCCESSURL =
		"# an optional name value (leading the to address) " +
		"that overrides the default value set by the system administrator.";
	private final static String COMMENT_ERRORURL =
		"# an optional address value that overrides the " +
		"default \"To Address\" set by the system administrator.";
	private final static String COMMENT_TEMPLATE =
		"# the name of a mail template file that shall be " +
		"processed (file name only, without path)";

	/**
	 * Writes the individual property file of the given provider. If this
	 * provider has a set provider directory property, the value of this
	 * property is uses as the file name, else a generic one is used and
	 * stored within the app's config directory.
	 *
	 * @param provider the provider to store the individual property file for
	 *
	 * @throws MailWebFormException if the config dir property isn't set
	 * @throws IOException if the file can't be created and/or written somehow
	 */
	void writeFile(MailFormProvider provider)
		throws MailWebFormException, IOException
	{
		//append individual descriptions and settings
		StringBuffer sb = new StringBuffer();
		sb.append(getPropertyDescriptor(COMMENT_SMTPHOST, SMTP_HOST,
				provider.getIndividualSMTPHost()));
		sb.append(getPropertyDescriptor(COMMENT_SMTPPORT, SMTP_PORT,
				"" + provider.getIndividualSMTPPort()));
		sb.append(getPropertyDescriptor(COMMENT_TONAME, TONAME,
				provider.getIndividualToName()));
		sb.append(getPropertyDescriptor(COMMENT_TOADDRESS, TOADDRESS,
				provider.getIndividualToAddress()));
		sb.append(getPropertyDescriptor(COMMENT_SUCCESSURL, SUCCESS_URL,
				provider.getIndividualSuccessURL()));
		sb.append(getPropertyDescriptor(COMMENT_ERRORURL, ERROR_URL,
				provider.getIndividualErrorURL()));
		sb.append(getPropertyDescriptor(COMMENT_TEMPLATE, TEMPLATE,
				provider.getIndividualTemplate()));

		//write file
		File propertyFile = new File((!StringUtils.isEmpty(
					provider.getProviderDirectory()))
				? (provider.getProviderDirectory() + provider.getName() +
				PROPERTY_SUFFIX)
				: (AbstractMailWebFormServlet.getConfigDirProperty() +
				provider.getName() + PROPERTY_SUFFIX));
		BufferedWriter bw = new BufferedWriter(new FileWriter(propertyFile));
		bw.write(sb.toString());
		bw.flush();
		bw.close();
	}

	/**
	 * Builds one property descriptor, consisting of a comment line and one
	 * with the key/value pair, separated by a '='. If the given value is
	 * null, nothing is set as the property value and the line is commented
	 * out.
	 *
	 * @param sComment the comment of the property
	 * @param sKey the property key
	 * @param sValue the property value (may be null)
	 *
	 * @return the fully configured string, ending with two newline characters
	 */
	private String getPropertyDescriptor(String sComment, String sKey,
		String sValue)
	{
		StringBuffer sb = new StringBuffer();
		sb.append(sComment).append(NEWLINE);

		if (StringUtils.isEmpty(sValue)) {
			sb.append('#').append(sKey).append('=');
		} else {
			sb.append(sKey).append('=').append(sValue);
		}

		return sb.append(NEWLINE).append(NEWLINE).toString();
	}
}
