package org.alltimeflashdreamer.mailwebform;

import org.alltimeflashdreamer.util.StringUtils;

import java.io.Serializable;

import javax.servlet.http.HttpServletRequest;


/**
 * Title: JSPBean<br>
 * Description: This simple bean is designed to provide the constant and
 * dynamic values used in the various JSP files. To be able to serve the
 * various different JSP sites appropriately, the class contains a series of
 * constructors that are designed to get the parameters used by the follower
 * site(s).
 *
 * @author Copyright (c) 2001-2004 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
public class JSPBean implements MailWebFormConstants, Serializable
{
	private String sErrorMessage = "";
	private boolean bAccountWasNewlyCreated = false;
	private String[] sProviders = new String[0];
	private String sProviderName = "";
	private String sAdminUserName = null;
	private GlobalProperties globalProperties = null;
	private MailFormProviderList mailFormProviders = null;

	/**
	 * (inactive)
	 */
	public JSPBean() {}

	/**
	 * Constructor simply stores the fully rendered error message that consists
	 * of the error message of the given exception itself as well as those of
	 * the cascaded root cause(s), if any, ready to be shown on the error JSP
	 * site.
	 *
	 * @param mwfe the error to be rendered
	 */
	JSPBean(MailWebFormException mwfe)
	{
		this.sErrorMessage = mwfe.getMessage();

		Throwable cause = mwfe.getCause();

		while (cause != null) {
			sErrorMessage += ("<br />" + cause.getMessage());
			cause = cause.getCause();
		}
	}

	/**
	 * Constructor simply stores the given string. If it's null, this indicates
	 * that not the admin credentials, but the password of a provider shall be
	 * changed.
	 *
	 * @param sProviderName the name of the current provider (empty string in
	 * 		  case of the adding of a new one)
	 * @param sAdminUserName the user name of the admin account or null
	 */
	JSPBean(String sProviderName, String sAdminUserName)
	{
		this.sProviderName = sProviderName;
		this.sAdminUserName = sAdminUserName;
	}

	/**
	 * Constructor simply stores the given boolean value, ready to be used on
	 * the main JSP site.
	 *
	 * @param bAccountWasNewlyCreated whether the account was newly created and
	 * 		  the related message box shall be shown or not
	 * @param sProviders the list of providers to be shown in the select box
	 */
	JSPBean(boolean bAccountWasNewlyCreated, String[] sProviders)
	{
		this.bAccountWasNewlyCreated = bAccountWasNewlyCreated;
		this.sProviders = sProviders;
	}

	/**
	 * Constructor simply stores the given values, ready to be used on the one
	 * of the add/edit/edit-system-provider JSP sites.
	 *
	 * @param sProviderName the name of the current provider (empty string in
	 * 		  case of the adding of a new one)
	 * @param globalProperties the list of the global app properties
	 * @param mailFormProviders the provider list, holding the references to
	 * 		  the various providers and their properties
	 */
	JSPBean(String sProviderName, GlobalProperties globalProperties,
		MailFormProviderList mailFormProviders)
	{
		this.sProviderName = sProviderName;
		this.globalProperties = globalProperties;
		this.mailFormProviders = mailFormProviders;
	}

	/**
	 * Returns the specified provider instance or null, if the provider list
	 * hasn't been passed in the constructor or the specified provider name is
	 * not registered.
	 *
	 * @param sProviderName the name of the provider to be returned
	 *
	 * @return the specified provider instance or null
	 */
	public MailFormProvider getMailFormProvider(String sProviderName)
	{
		if (mailFormProviders != null) {
			return mailFormProviders.getProvider(sProviderName);
		} else {
			return null;
		}
	}

	/**
	 * Returns the error message, as received in the constructor, or an empty
	 * string, if not provided.
	 *
	 * @return the error message or an empty string
	 */
	public String getErrorMessage()
	{
		return sErrorMessage;
	}

	/**
	 * Returns whether the account was newly created or not.
	 *
	 * @return show "successfully created account" message box?
	 */
	public boolean isNewlyCreatedAccount()
	{
		return bAccountWasNewlyCreated;
	}

	/**
	 * Returns the title to be shown on the login page of the admin tool.
	 *
	 * @return the (admin) login title
	 */
	public String getLoginTitle()
	{
		return APPNAME + " - Admin Tool: Login Screen";
	}

	/**
	 * Returns the title to be shown on the error page of the admin tool.
	 *
	 * @return the error title
	 */
	public String getErrorTitle()
	{
		return APPNAME + " - Admin Tool: Error Page";
	}

	/**
	 * Returns the title to be shown on the main page of the admin tool.
	 *
	 * @return the main title
	 */
	public String getMainTitle()
	{
		return APPNAME + " - Admin Tool: Main Window";
	}

	/**
	 * Returns the title to be shown on the edit pages of the admin tool.
	 *
	 * @return the edit title
	 */
	public String getEditTitle()
	{
		return APPNAME + " - Admin Tool: Provider Settings";
	}

	/**
	 * Returns the title to be shown on the change password/credentials pages
	 * of the admin tool.
	 *
	 * @param bPasswordOnly whether only the password shall be changed or not
	 * 		  (i.e. the whole [admin] credentials)
	 *
	 * @return the change title
	 */
	public String getChangeTitle(boolean bPasswordOnly)
	{
		return APPNAME + " - Admin Tool: Change " +
		((bPasswordOnly) ? "Password" : "Credentials");
	}

	/**
	 * Returns the list of providers, as received in the constructor, or an
	 * empty list, if another constructor was called.
	 *
	 * @return the list of elements to be shown in the HTML select box
	 */
	public String[] getProviderList()
	{
		return sProviders;
	}

	/**
	 * Returns the name of the currently active provider or an empty string, if
	 * not set or a new one is to be created.
	 *
	 * @return the current provider's name or an empty string
	 */
	public String getProviderName()
	{
		return sProviderName;
	}

	/**
	 * Returns whether currently a new provider is set up, i.e. whether a
	 * provider name is defined in here or not.
	 *
	 * @return adding new provider (on edit page)?
	 */
	public boolean isNewProvider()
	{
		return StringUtils.isEmpty(sProviderName);
	}

	/**
	 * Returns whether currently the system provider's settings are edited or
	 * not.
	 *
	 * @return current provider is the system provider?
	 */
	public boolean isSystemProvider()
	{
		return MailFormProvider.isSystemProvider(sProviderName);
	}

	/**
	 * Returns the Properties implementation holding the application's global
	 * settings.
	 *
	 * @return the list of the global app properties
	 */
	public GlobalProperties getGlobalProperties()
	{
		return globalProperties;
	}

	/**
	 * Returns the mail form provider list implementation.
	 *
	 * @return the provider list, holding the references to the various
	 * 		   providers and their properties
	 */
	public MailFormProviderList getMailFormProviders()
	{
		return mailFormProviders;
	}

	/**
	 * Returns whether the currently logged in user is the admin user or not.
	 *
	 * @param request where to read the current-user session property from
	 *
	 * @return whether this user is the application admin or not
	 */
	public boolean isAdmin(HttpServletRequest request)
	{
		return ((String)request.getSession().getAttribute(CURRENTUSER)).equalsIgnoreCase(ADMIN);
	}

	/**
	 * Returns whether the user is currently changing the admin credentials or
	 * not.
	 *
	 * @return client is on change-admin-credentials page?
	 */
	public boolean isChangingAdminCredentials()
	{
		return sAdminUserName != null;
	}

	/**
	 * Returns the user name of the admin account, if set in the relative
	 * constructor, or null.
	 *
	 * @return the admin user name or null
	 */
	public String getAdminUserName()
	{
		return sAdminUserName;
	}
}
