package org.alltimeflashdreamer.mailwebform;

import org.alltimeflashdreamer.util.StringUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.Serializable;

import java.util.Properties;


/**
 * Title: MailFormProvider<br>
 * Description: This class stores the values for one registered mailform
 * provider. It is able to return the each separate as well as the highest
 * active setting per type (see description of MailWebFormServlet) and reload
 * the property files dynamically.
 *
 * @author Copyright (c) 2001-2004 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
public class MailFormProvider implements MailWebFormConstants, Serializable
{
	private String sName;
	private String sProviderDirectory;
	private String sDefaultToName;
	private String sToName;
	private String sDefaultToAddress;
	private String sToAddress;
	private String sDefaultSMTPHost;
	private String sSMTPHost;
	private int iDefaultSMTPPort;
	private int iSMTPPort;
	private String sDefaultSuccessURL;
	private String sSuccessURL;
	private String sDefaultErrorURL;
	private String sErrorURL;
	private String sDefaultTemplate;
	private String sTemplate;
	private long lLastModified = 0L;
	private boolean bIsSMTPOverwritingAllowed;
	private String sPassword;

	/**
	 * The constructor receives the name of the current mailform provider
	 * together with the pointer to the ResourceBundle where the default
	 * settings are stored.
	 *
	 * @param sName the name fo this provider
	 * @param globalProperties the list containing the global properties of the
	 * 		  MailWebForm application
	 *
	 * @throws MailWebFormException thrown on problems with the config dir
	 * 		   property
	 * @throws IOException in case an I/O error occurred while accessing the
	 * 		   provider's properties file
	 */
	MailFormProvider(String sName, GlobalProperties globalProperties)
		throws MailWebFormException, IOException
	{
		//read values defined in the global resource file
		this.sName = sName;
		setProviderDirectory(globalProperties.getProperty(getSettingKey(
					PROVIDERDIRECTORY)));
		this.sDefaultToName = globalProperties.getProperty(MAILFORMPROVIDER +
				"." + sName + "." + TONAME);
		this.sDefaultToAddress = globalProperties.getProperty(MAILFORMPROVIDER +
				"." + sName + "." + TOADDRESS);
		this.sDefaultSMTPHost = globalProperties.getSMTPHost();
		this.iDefaultSMTPPort = globalProperties.getSMTPPort();
		this.sDefaultSuccessURL = globalProperties.getSuccessURL();
		this.sDefaultErrorURL = globalProperties.getErrorURL();
		setSMTPOverwritingAllowed(globalProperties.getProperty(getSettingKey(
					SMTP_ALLOWOVERWRITE)));
		this.sPassword = globalProperties.getProperty(getSettingKey(PASSWORD));
		this.sDefaultTemplate = globalProperties.getProperty(MAILFORMPROVIDER +
				"." + sName + "." + TEMPLATE);

		//check whether there is an individual property file; if so, read it
		//and store the values
		readIndividualProperties();
	}

	/**
	 * Sets/Updates the "global" properties of this provider after having
	 * verified the input.
	 *
	 * @param sProviderDirectory the new setting for the path to the provider's
	 * 		  individual files (overwriting property file and mail template)
	 * @param sToName the new setting for the "master" to-name
	 * @param sToAddress the new setting for the "master" to-address
	 * @param sSMTPOverwritingAllowed the new setting for the allowing of
	 * 		  overwrite (if null, then the default value is used which is
	 * 		  true)
	 * @param sTemplate the name of the mail template to be used
	 */
	void updateGlobalSettings(String sProviderDirectory, String sToName,
		String sToAddress, String sSMTPOverwritingAllowed, String sTemplate)
	{
		setProviderDirectory(sProviderDirectory);
		this.sDefaultToName = sToName;
		this.sDefaultToAddress = sToAddress;
		setSMTPOverwritingAllowed(sSMTPOverwritingAllowed);
		this.sDefaultTemplate = sTemplate;
	}

	/**
	 * Sets/Updates the password property of this provider.
	 *
	 * @param sPassword the encoded password that the provider uses to log in
	 * 		  into the admin tool
	 */
	void updatePassword(String sPassword)
	{
		this.sPassword = sPassword;
	}

	/**
	 * Sets/Updates the individual properties of this provider after having
	 * verified the input. Note: The SMTP host and port properties may be set
	 * here, but the "master" setting may prohibit the their activation.
	 *
	 * @param sToName the new setting for the individual to-name
	 * @param sToAddress the new setting for the individual to-address
	 * @param sSMTPHost the new setting for the SMTP host
	 * @param iSMTPPort the new setting for the SMTP port
	 * @param sSuccessURL the new setting of the site to be shown after having
	 * 		  successfully sent a mail
	 * @param sErrorURL the new setting of the site to be shown in case an
	 * 		  error occurred while trying to send a mail
	 * @param sTemplate the name of the mail template to be used
	 */
	void updateIndividualSettings(String sToName, String sToAddress,
		String sSMTPHost, int iSMTPPort, String sSuccessURL, String sErrorURL,
		String sTemplate)
	{
		this.sToName = sToName;
		this.sToAddress = sToAddress;
		this.sSMTPHost = sSMTPHost;
		this.iSMTPPort = (iSMTPPort >= 0) ? iSMTPPort : 0;
		this.sSuccessURL = sSuccessURL;
		this.sErrorURL = sErrorURL;
		this.sTemplate = sTemplate;
	}

	/**
	 * Returns the name of this provider.
	 *
	 * @return as received in the constructor
	 */
	public String getName()
	{
		return sName;
	}

	/**
	 * Returns the value of the provider's individual path property.
	 *
	 * @return as read from the list of the provider-specific properties
	 * 		   (&quot;&quot;, if not set)
	 */
	public String getProviderDirectory()
	{
		return (sProviderDirectory == null) ? "" : sProviderDirectory;
	}

	/**
	 * Returns whether the overwriting of the default SMTP server is allowed
	 * for this provider or not.
	 *
	 * @return as read from the list of the provider-specific properties
	 */
	public boolean isSMTPOverwritingAllowed()
	{
		return bIsSMTPOverwritingAllowed;
	}

	/**
	 * Returns the encoded password that the provider uses to log in into the
	 * admin tool.
	 *
	 * @return the MD5 hash of the provider's password (&quot;&quot;, if not
	 * 		   set)
	 */
	public String getPassword()
	{
		return (sPassword == null) ? "" : sPassword;
	}

	/**
	 * Checks whether it's necessary to read/update the provider-specific
	 * settings. If so, the SMTP host and port, the success and the error url
	 * are read. The SMTP host, SMTP port and to-address settings are only
	 * updated if the provider is allowed to do this.
	 *
	 * @throws MailWebFormException in case there are problems accessing the
	 * 		   file or the config dir property isn't defined (yet)
	 */
	void readIndividualProperties() throws MailWebFormException
	{
		//don't do this for the system provider...
		if (isSystemProvider()) {
			return;
		}

		//get the "wished" property file
		File propertyFile = getPropertyFile();

		//if it exists and/or has been touched since the last access of this method, (re)read it
		if ((propertyFile != null) && propertyFile.exists()) {
			if (this.lLastModified != propertyFile.lastModified()) {
				Properties ownProperties = new Properties();

				synchronized (propertyFile) {
					this.lLastModified = propertyFile.lastModified();

					FileInputStream fis = null;

					try {
						fis = new FileInputStream(propertyFile);
						ownProperties.load(new FileInputStream(propertyFile));
					} catch (IOException ioe) {
						throw new MailWebFormException(
							"Error while reading the provider's " +
							"individual property file!", ioe);
					} finally {
						if (fis != null) {
							try {
								fis.close();
							} catch (Exception ex) {}
						}
					}
				}

				if (this.bIsSMTPOverwritingAllowed) {
					this.sSMTPHost = ownProperties.getProperty(SMTP_HOST);

					try {
						this.iSMTPPort = new Integer(ownProperties.getProperty(
									SMTP_PORT)).intValue();
					} catch (NumberFormatException nfe) {
						this.iSMTPPort = 0;
					}
				} else {
					this.sSMTPHost = null;
					this.iSMTPPort = 0;
				}

				this.sSuccessURL = ownProperties.getProperty(SUCCESS_URL);
				this.sErrorURL = ownProperties.getProperty(ERROR_URL);
				this.sToName = ownProperties.getProperty(TONAME);
				this.sToAddress = ownProperties.getProperty(TOADDRESS);
				this.sTemplate = ownProperties.getProperty(TEMPLATE);
			}
		} else { //else unset each pointer
			this.unsetOwnSettings();
		}
	}

	/**
	 * Returns the default to-name setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getDefaultToName()
	{
		return (sDefaultToName == null) ? "" : sDefaultToName;
	}

	/**
	 * Returns the individual to-name setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getIndividualToName()
	{
		return (sToName == null) ? "" : sToName;
	}

	/**
	 * Returns the individual or the default to-name setting or null, if both
	 * aren't defined.
	 *
	 * @return may be null or "", if not set
	 */
	public String getToName()
	{
		return MailFormProviderList.getValidProperty(sToName, sDefaultToName);
	}

	/**
	 * Returns the default to-address setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getDefaultToAddress()
	{
		return (sDefaultToAddress == null) ? "" : sDefaultToAddress;
	}

	/**
	 * Returns the individual to-address setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getIndividualToAddress()
	{
		return (sToAddress == null) ? "" : sToAddress;
	}

	/**
	 * Returns the individual or the default to-address setting or null, if
	 * both aren't defined.
	 *
	 * @return may be null or "", if not set
	 */
	public String getToAddress()
	{
		return MailFormProviderList.getValidProperty(sToAddress,
			sDefaultToAddress);
	}

	/**
	 * Returns the default SMTP host setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getDefaultSMTPHost()
	{
		return (sDefaultSMTPHost == null) ? "" : sDefaultSMTPHost;
	}

	/**
	 * Returns the individual SMTP host setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getIndividualSMTPHost()
	{
		return (sSMTPHost == null) ? "" : sSMTPHost;
	}

	/**
	 * Returns the individual or the SMTP host setting or null, if both aren't
	 * defined.
	 *
	 * @return may be null or "", if not set
	 */
	public String getSMTPHost()
	{
		if (!bIsSMTPOverwritingAllowed) {
			return this.sDefaultSMTPHost;
		}

		return MailFormProviderList.getValidProperty(sSMTPHost, sDefaultSMTPHost);
	}

	/**
	 * Returns the default SMTP port setting.
	 *
	 * @return may be null or "", if not set
	 */
	public int getDefaultSMTPPort()
	{
		return iDefaultSMTPPort;
	}

	/**
	 * Returns the individual SMTP host setting.
	 *
	 * @return may be null or "", if not set
	 */
	public int getIndividualSMTPPort()
	{
		return iSMTPPort;
	}

	/**
	 * Returns the SMTP port setting.
	 *
	 * @return may be null or "", if not set
	 */
	public int getSMTPPort()
	{
		if (!bIsSMTPOverwritingAllowed) {
			return this.iDefaultSMTPPort;
		}

		return ((this.iSMTPPort != 0) ? this.iSMTPPort : this.iDefaultSMTPPort);
	}

	/**
	 * Returns the default success url setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getDefaultSuccessURL()
	{
		return (sDefaultSuccessURL == null) ? "" : sDefaultSuccessURL;
	}

	/**
	 * Returns the individual success url setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getIndividualSuccessURL()
	{
		return (sSuccessURL == null) ? "" : sSuccessURL;
	}

	/**
	 * Returns the success url setting.
	 *
	 * @return may be null or "", if not set
	 */
	public String getSuccessURL()
	{
		return MailFormProviderList.getValidProperty(sSuccessURL,
			sDefaultSuccessURL);
	}

	/**
	 * Returns the default error url setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getDefaultErrorURL()
	{
		return (sDefaultErrorURL == null) ? "" : sDefaultErrorURL;
	}

	/**
	 * Returns the individual error url setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getIndividualErrorURL()
	{
		return (sErrorURL == null) ? "" : sErrorURL;
	}

	/**
	 * Returns the error url setting.
	 *
	 * @return may be null or "", if not set
	 */
	public String getErrorURL()
	{
		return MailFormProviderList.getValidProperty(sErrorURL, sDefaultErrorURL);
	}

	/**
	 * Returns the default template setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getDefaultTemplate()
	{
		return (sDefaultTemplate == null) ? "" : sDefaultTemplate;
	}

	/**
	 * Returns the individual template setting.
	 *
	 * @return &quot;&quot;, if not set
	 */
	public String getIndividualTemplate()
	{
		return (sTemplate == null) ? "" : sTemplate;
	}

	/**
	 * Returns the template setting.
	 *
	 * @return may be null or "", if not set
	 */
	public String getTemplate()
	{
		return MailFormProviderList.getValidProperty(sTemplate, sDefaultTemplate);
	}

	/**
	 * Returns an array filled with strings formatted like
	 * "&lt;key&gt;=&lt;value&gt;", ready to be written into the global
	 * properties file, together with the description of the section wherein
	 * these strings are located in the file.
	 *
	 * @return list to be copied as-is into the global properties file
	 */
	String[] getGlobalPropertiesPart()
	{
		String[] sReturnValue = null;

		if (isSystemProvider()) {
			sReturnValue = new String[3];
			sReturnValue[1] = getSettingKey(TONAME) + "=" + getDefaultToName();
			sReturnValue[2] = getSettingKey(TOADDRESS) + "=" +
				getDefaultToAddress();
		} else {
			sReturnValue = new String[7];
			sReturnValue[1] = getSettingKey(PROVIDERDIRECTORY) + "=" +
				getProviderDirectory();
			sReturnValue[2] = getSettingKey(TONAME) + "=" + getDefaultToName();
			sReturnValue[3] = getSettingKey(TOADDRESS) + "=" +
				getDefaultToAddress();
			sReturnValue[4] = getSettingKey(SMTP_ALLOWOVERWRITE) + "=" +
				((this.bIsSMTPOverwritingAllowed) ? "YES" : "NO");
			sReturnValue[5] = getSettingKey(PASSWORD) + "=" + getPassword();
			sReturnValue[6] = getSettingKey(TEMPLATE) + "=" +
				getDefaultTemplate();
		}

		sReturnValue[0] = "# settings of provider \"" + sName + "\":";

		return sReturnValue;
	}

	/**
	 * Returns whether this provider is the system provider or not.
	 *
	 * @return is this the system provider instance?
	 */
	boolean isSystemProvider()
	{
		return isSystemProvider(sName);
	}

	/**
	 * Returns whether the given provider name is the one of the system
	 * provider or not.
	 *
	 * @param sProviderName the provider name to be checked
	 *
	 * @return equals "system" (case-insensitive check)?
	 */
	static boolean isSystemProvider(String sProviderName)
	{
		return (!StringUtils.isEmpty(sProviderName))
		? sProviderName.equalsIgnoreCase(SYSTEMPROVIDER) : false;
	}

	/**
	 * Returns the beginning of each setting that is specific for this
	 * MailFormProvider instance.
	 *
	 * @return the MAILFORMPROVIDER constant, followed by a dot and the
	 * 		   provider name
	 */
	private String getSettingStart()
	{
		return MAILFORMPROVIDER + "." + sName;
	}

	/**
	 * Appends a dot and the given setting identifier to the return value of
	 * the getSettingStart method.
	 *
	 * @param sWhichSetting the end of the setting, to be appended to the
	 * 		  setting start
	 *
	 * @return the fully formatted setting key
	 */
	private String getSettingKey(String sWhichSetting)
	{
		return getSettingStart() + "." + sWhichSetting;
	}

	/**
	 * Unsets the provider-specific values.
	 */
	private void unsetOwnSettings()
	{
		this.lLastModified = 0L;
		this.sToName = "";
		this.sToAddress = "";
		this.sSMTPHost = "";
		this.iSMTPPort = 0;
		this.sSuccessURL = "";
		this.sErrorURL = "";
		this.sTemplate = "";
	}

	/**
	 * Returns the abstract File instance of the provider's individual property
	 * file. This file lies either in the config directory or in the
	 * provider's individual directory, if defined in the provider's global
	 * properties.
	 *
	 * @return the abstract File instance of the providers individual property
	 * 		   file (no check is done against existence in here!)
	 *
	 * @throws MailWebFormException if the config dir property isn't defined
	 * 		   (yet)
	 */
	private File getPropertyFile() throws MailWebFormException
	{
		File propertyFile = null;

		if (!StringUtils.isEmpty(sProviderDirectory)) {
			propertyFile = new File(sProviderDirectory + sName +
					PROPERTY_SUFFIX);
		} else {
			propertyFile = new File(AbstractMailWebFormServlet.getConfigDirProperty() +
					sName + PROPERTY_SUFFIX);
		}

		return propertyFile;
	}

	/**
	 * Verifies and sets the provider directory property.
	 *
	 * @param sProviderDirectory the new path to be set
	 */
	private void setProviderDirectory(String sProviderDirectory)
	{
		if (!StringUtils.isEmpty(sProviderDirectory)) {
			sProviderDirectory = StringUtils.appendFileSeparator(StringUtils.replaceBackslashes(
						sProviderDirectory), true);
		}

		this.sProviderDirectory = sProviderDirectory;
	}

	/**
	 * Verifies and sets the is-smtp-overwriting-allowed property.
	 *
	 * @param sSMTPOverwritingAllowed the String describing the new  boolean
	 * 		  property to be set
	 */
	private void setSMTPOverwritingAllowed(String sSMTPOverwritingAllowed)
	{
		if ((sSMTPOverwritingAllowed == null) ||
				(sSMTPOverwritingAllowed.equalsIgnoreCase("NO")) ||
				(sSMTPOverwritingAllowed.equalsIgnoreCase("DENY"))) {
			this.bIsSMTPOverwritingAllowed = false;
		} else {
			this.bIsSMTPOverwritingAllowed = true;
		}
	}
}
