package org.alltimeflashdreamer.mailwebform;

import java.io.IOException;
import java.io.Serializable;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.StringTokenizer;

import org.alltimeflashdreamer.util.StringUtils;


/**
 * Title: MailFormProviderList<br>
 * Description: Represents a handler of a list of known/defined providers of a
 * mailwebform site, as defined in the application's global properties file.
 *
 * @author Copyright (c) 2001-2004 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
public class MailFormProviderList implements Serializable
{
	/** Stores the registered mailwebform providers */
	private Hashtable providerList;

	/**
	 * The constructor receives the Properties object where the providers'
	 * settings are stored, especially the comma-separated list of
	 * defined/active providers which is used to build the internal list of
	 * providers
	 *
	 * @param globalProperties a Properties object which loaded the data of the
	 * 		  global properties file before
	 *
	 * @throws MailWebFormException thrown on problems with the config dir
	 * 		   property
	 * @throws IOException in case an I/O error occurred during the accessing
	 * 		   of a provider's own properties file
	 */
	MailFormProviderList(GlobalProperties globalProperties)
		throws MailWebFormException, IOException
	{
		String sMailFormProviders = globalProperties.getProperty(MailWebFormConstants.MAILFORMPROVIDERLIST);
		StringTokenizer st = new StringTokenizer(sMailFormProviders, ",");
		providerList = new Hashtable();

		while (st.hasMoreTokens()) {
			String sNextProviderName = st.nextToken().trim();
			providerList.put(sNextProviderName.toLowerCase(),
				new MailFormProvider(sNextProviderName, globalProperties));
		}
	}

	/**
	 * Returns a String[] containing the names of the registered providers.
	 *
	 * @return the list of registered provider names, at least filled with the
	 * 		   "system" one
	 */
	String[] getProviderNames()
	{
		String[] sProviderNames = new String[providerList.size()];
		int iCtr = 0;

		for (Iterator iterValues = providerList.values().iterator(); iterValues.hasNext();) {
			sProviderNames[iCtr++] = ((MailFormProvider)iterValues.next()).getName();
		}

		Arrays.sort(sProviderNames);

		return sProviderNames;
	}

	/**
	 * Returns whether the given provider name is in the list of all registered
	 * providers or not (case-insensitive check).
	 *
	 * @param sProviderName the provider name to search for
	 *
	 * @return whether this provider is a registered one or not
	 */
	boolean hasProvider(String sProviderName)
	{
		return providerList.containsKey(sProviderName.toLowerCase());
	}

	/**
	 * Returns the specified provider instance or null, if the specified
	 * provider name is not registered (case-insensitive check).
	 *
	 * @param sProviderName the name of the provider to be returned
	 *
	 * @return the specified provider instance or null
	 */
	MailFormProvider getProvider(String sProviderName)
	{
		return (MailFormProvider)providerList.get(sProviderName.toLowerCase());
	}

	/**
	 * Returns the ProviderDirectory parameter of the given provider or null,
	 * if the specified provider doesn't exist.
	 *
	 * @param sProviderName the name of the provider
	 *
	 * @return the path of the directory where the provider's own properties
	 * 		   file and/or the mail template are stored
	 */
	public String getProviderDirectory(String sProviderName)
	{
		MailFormProvider provider = getProvider(sProviderName);

		return ((provider == null) ? null : provider.getProviderDirectory());
	}

	/**
	 * Returns the list of lines that are to be written into the global
	 * properties file for the given provider. The return value can be used
	 * as-is to write it into the output file.
	 *
	 * @param sProviderName the name of the provider
	 *
	 * @return the list of "ready to write" lines (may be null)
	 */
	String[] getGlobalPropertiesPart(String sProviderName)
	{
		MailFormProvider provider = getProvider(sProviderName);

		return ((provider == null) ? null : provider.getGlobalPropertiesPart());
	}

	/**
	 * Returns the given provider's "SMTP_HOST" setting.
	 *
	 * @param sProviderName the name of the provider
	 * @param sDefaultSMTPHost the value to use in case the provider is not
	 * 		  known
	 *
	 * @return the provider's own SMTP host or the given default SMTP host.
	 */
	String getSMTPHost(String sProviderName, String sDefaultSMTPHost)
	{
		MailFormProvider provider = getProvider(sProviderName);

		if (provider == null) {
			return sDefaultSMTPHost;
		}

		return getValidProperty(provider.getSMTPHost(), sDefaultSMTPHost);
	}

	/**
	 * Returns the given provider's "SMTP_PORT" setting.
	 *
	 * @param sProviderName the name of the provider
	 * @param iDefaultSMTPPort the value to use in case the provider is not
	 * 		  known
	 *
	 * @return the provider's own SMTP port or the given default SMTP Port.
	 */
	int getSMTPPort(String sProviderName, int iDefaultSMTPPort)
	{
		MailFormProvider provider = getProvider(sProviderName);

		if (provider == null) {
			return iDefaultSMTPPort;
		}

		return ((provider.getSMTPPort() == 0) ? iDefaultSMTPPort
											  : provider.getSMTPPort());
	}

	/**
	 * Returns the given provider's "SUCCESS_URL" setting.
	 *
	 * @param sProviderName the name of the provider
	 * @param sDefaultSuccessURL the value to use in case the provider is not
	 * 		  known
	 *
	 * @return the provider's own success URL or the given default success URL.
	 */
	String getSuccessURL(String sProviderName, String sDefaultSuccessURL)
	{
		MailFormProvider provider = getProvider(sProviderName);

		if (provider == null) {
			return sDefaultSuccessURL;
		}

		return getValidProperty(provider.getSuccessURL(), sDefaultSuccessURL);
	}

	/**
	 * Returns the given provider's "ERROR_URL" setting.
	 *
	 * @param sProviderName the name of the provider
	 * @param sDefaultErrorURL the value to use in case the provider is not
	 * 		  known
	 *
	 * @return the provider's own error URL or the given default error URL.
	 */
	String getErrorURL(String sProviderName, String sDefaultErrorURL)
	{
		MailFormProvider provider = getProvider(sProviderName);

		if (provider == null) {
			return sDefaultErrorURL;
		}

		return getValidProperty(provider.getErrorURL(), sDefaultErrorURL);
	}

	/**
	 * Returns the given provider's "TONAME" setting.
	 *
	 * @param sProviderName the name of the provider
	 * @param sDefaultToName the value to use in case the provider is not known
	 *
	 * @return the provider's mail address name setting or the given default
	 * 		   to-name
	 */
	public String getToName(String sProviderName, String sDefaultToName)
	{
		MailFormProvider provider = getProvider(sProviderName);

		if (provider == null) {
			return sDefaultToName;
		}

		return getValidProperty(provider.getToName(), sDefaultToName);
	}

	/**
	 * Returns the given provider's "TOADDRESS" setting.
	 *
	 * @param sProviderName the name of the provider
	 * @param sDefaultToAddress the value to use in case the provider is not
	 * 		  known
	 *
	 * @return the provider's mail address setting or the given default
	 * 		   to-address
	 */
	public String getToAddress(String sProviderName, String sDefaultToAddress)
	{
		MailFormProvider provider = getProvider(sProviderName);

		if (provider == null) {
			return sDefaultToAddress;
		}

		return getValidProperty(provider.getToAddress(), sDefaultToAddress);
	}

	/**
	 * Returns the boolean value of the given provider's "SMTP_ALLOWOVERWRITE"
	 * setting. If this setting is not defined, the default setting (which is
	 * true) is returned.
	 *
	 * @param sProviderName the name of the provider
	 *
	 * @return the provider's allow SMTP overwrite setting
	 */
	public boolean isSMTPOverwriteAllowed(String sProviderName)
	{
		MailFormProvider provider = getProvider(sProviderName);

		return ((provider == null) ? true : provider.isSMTPOverwritingAllowed());
	}

	/**
	 * Returns the password parameter of the given provider or null, if the
	 * specified provider doesn't exist.
	 *
	 * @param sProviderName the name of the provider
	 *
	 * @return the encoded admin tool password of the provider
	 */
	public String getPassword(String sProviderName)
	{
		MailFormProvider provider = getProvider(sProviderName);

		return ((provider == null) ? null : provider.getPassword());
	}

	/**
	 * Steps through the list of registered providers and forces each one to
	 * check whether the provider-specific settings file has been changed
	 * since the last access. If so, the properties of this provider are
	 * re-read.
	 *
	 * @throws MailWebFormException in case there are problems accessing the
	 * 		   file or the config dir property isn't defined (yet)
	 */
	void checkProviderSettings() throws MailWebFormException
	{
		for (Iterator iterValues = providerList.values().iterator(); iterValues.hasNext();) {
			((MailFormProvider)iterValues.next()).readIndividualProperties();
		}
	}

	/**
	 * Searches the given provider in the list and forces him to reread the
	 * settings from the individual property file. <i>It is recommended to
	 * call this method</i> before each querying of the provider's settings!
	 *
	 * @param sProviderName name of the provider, to find its instance
	 *
	 * @throws MailWebFormException in case there are problems accessing the
	 * 		   file or the config dir property isn't defined (yet)
	 */
	void checkProviderSettings(String sProviderName)
		throws MailWebFormException
	{
		((MailFormProvider)providerList.get(sProviderName.toLowerCase())).readIndividualProperties();
	}

	/**
	 * Updates the "personal" setting of the specified provider by recreating
	 * the MailFormProvider instance and overwriting the old one within this
	 * list.
	 *
	 * @param sProviderName update whom?
	 * @param sProviderDirectory the new setting for the path to the provider's
	 * 		  individual files (overwriting property file and mail template)
	 * @param sToName the new setting for the "master" to-name
	 * @param sToAddress the new setting for the "master" to-address
	 * @param sAllowOverwrite the new setting for the allowing of overwrite (if
	 * 		  null, then the default value is used which is true)
	 * @param sTemplate the name of the mail template to be used
	 * @param globalProperties the list containing the global properties
	 *
	 * @throws MailWebFormException thrown on problems with the config dir
	 * 		   property
	 * @throws IOException in case problems occurred while accessing the
	 * 		   provider's properties file
	 */
	void updateProvider(String sProviderName, String sProviderDirectory,
		String sToName, String sToAddress, String sAllowOverwrite,
		String sTemplate, GlobalProperties globalProperties)
		throws MailWebFormException, IOException
	{
		//recreate the provider and overwrite the former one
		MailFormProvider provider = getProvider(sProviderName);

		if (provider == null) {
			provider = new MailFormProvider(sProviderName, globalProperties);
			providerList.put(sProviderName.toLowerCase(), provider);
		}

		provider.updateGlobalSettings(sProviderDirectory, sToName, sToAddress,
			sAllowOverwrite, sTemplate);
	}

	/**
	 * Sets/Updates the password property of the specified provider.
	 *
	 * @param sProviderName update whom?
	 * @param sPassword the encoded password that the provider uses to log in
	 * 		  into the admin tool
	 */
	void updatePassword(String sProviderName, String sPassword)
	{
		MailFormProvider provider = getProvider(sProviderName);

		if (provider != null) {
			provider.updatePassword(sPassword);
		}
	}

	/**
	 * Removes the given provider from the list. If the input is not the name
	 * of a registered provider, nothing is done.
	 *
	 * @param sProviderName remove whom?
	 */
	void removeProvider(String sProviderName)
	{
		//check the input against length > 0
		if (StringUtils.isEmpty(sProviderName)) {
			return;
		}

		//check whether provider exists
		if (!hasProvider(sProviderName)) {
			return;
		}

		//remove the provider from the list; it's up to the caller to force
		//the rewriting of the underlying global properties file
		providerList.remove(sProviderName.toLowerCase());
	}

	/**
	 * Returns the given property, if not null and not empty, else the given
	 * default one.
	 *
	 * @param sProperty the - optional - individual property
	 * @param sDefaultProperty the fallback value, if the given property is
	 * 		  null or empty
	 *
	 * @return the valid, i.e. the "usable" property
	 */
	static String getValidProperty(String sProperty, String sDefaultProperty)
	{
		return (!StringUtils.isEmpty(sProperty)) ? sProperty : sDefaultProperty;
	}
}
