package org.alltimeflashdreamer.mailwebform;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.TreeMap;

import javax.mail.MessagingException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.alltimeflashdreamer.util.email.MailSender;
import org.alltimeflashdreamer.util.StringUtils;

/**
 * Title: MailWebFormServlet <br>
 * Description: This class represents the core of the MailWebForm application part. It collects the
 * required information to send a mail using the wrapper class
 * org.alltimeflashdreamer.util.email.MailSender. <br>
 * This information consists of the <i>required </i> and the <i>optional </i> parameters that are
 * provided by either the server administrator (lowest in hierarchy) or the MailWebForm provider
 * (using an overriding provider file) or the calling web site (highest priority; using URL
 * parameters from e.g. hidden form inputs). In case the information is set in more than one of
 * these sources, the source with the highest ranking is used. <br>
 * The required parameters are the host name and the port number of the SMTP server to send the
 * mails to as well as the sender's and the recipient's address (the recipient's name is recognized
 * by the servlet too, but not required by specification). <br>
 * Each additional URL parameter is treated as an optional parameter. They may be added to the
 * MailWebForm calling web site, usually an input mask to send a mail to the provider of a web site
 * (or someone else, but always to the same address). In case such additional/optional key-value
 * pairs are found in the request object, they are added at the beginning of the message text,
 * unless a mail template is defined. If one is defined, the placeholders in this template are
 * replaced with the values of the URL parameters having the same name as those placeholders. <br>
 * <i>Example </i>: Let's say one likes to have a gender checkbox on his input mask, and because he
 * uses two different input masks to send mails with, he wants to configure indipendent send mail
 * configurations. The solution for the first problem is to add a HTML input tag of type checkbox,
 * named e.g. "isMale", and a text telling the user that he shall activate the checkbox in case he
 * is man. The recipient will see a line "isMale = on" at the beginning of the mail, and no
 * additional line in case the box remained unchecked. The second solution is to configure two
 * different provider configurations in the admin tool and specify the send configuration using a
 * hidden HTML input tag named "provider" in both sites. The value of the first one would perhaps be
 * "myfirstsite" and the second one therefore "mysecondsite". This requires two files in the
 * providers overwriting files or in the application's config directory, one named
 * myfirstsite.properties and the second mysecondsite.properties, each of them having individually
 * configured overwrite parameters. <br>
 * For a HTML example, see the mailwebform_testexample.html file which is part of the distribution.
 * The important tags (required and optional) are demonstrated and commented there. <br>
 * The output which is produced after the sending of a site isn't designed at all. Therefore, one
 * probably wants to replace this output with his own site. The provider of the mail sending site
 * may show a site telling the user that the mail has been sent successfully (in case it was
 * successful) or one telling him that the mail couldn't be delivered (in case of some error).
 * However, he doesn't have to do this. In case these sites aren't defined for the current provider,
 * the application looks for default settings (defined for the "system" provider in the Admin Tool).
 * If those lack too, the servlet produces its own, ugly output. <br>
 * The system settings allow the definition of a site which is shown in case the application's
 * action method was called with illegal URL parameters, because some hacker was active or some
 * robot tried to enter or even spam the site. <br>
 * The decision which provider's settings are to be used to send a mail depends on the value of the
 * URL parameter "provider". If this value isn't defined, either the spam error site (if defined) or
 * an error message is shown by the servlet. If it's known, the settings for this provider are read
 * from the different sources, and for each setting, the highest existing (in ranking, see above) is
 * taken. Finally, the URL parameters named "subject" and "text" are received and their values added
 * to the generated mail which is then sent to the provider's own or the application's default SMTP
 * server. <br>
 * If this servlet is accessed without URL parameters, a simple default input mask is shown, as a
 * demonstration on the one and as a "is running" test on the other hand.
 * 
 * @author Copyright (c) 2001-2005 Florian Steinsiepe, Adrian Zaugg
 * @version 0.9
 */
public class MailWebFormServlet extends AbstractMailWebFormServlet
{
    private static final long serialVersionUID = 8982136561246285464L;

	/**
     * Looks for the URL parameters. If they're missing, the servlet is accessed for the first time,
     * and the default form is shown, where the user can enter the e-mail information. Otherwise, we
     * have the email info incoming, so we need to process it according to the caller's settings (if
     * any, else use the default settings again). <br>
     * Remember: The property files (both the global and the providers' ones) are always up to date.
     * Just touch them on the file system (i.e. change it's modification date), and they will be
     * reloaded the next time the servlet is accessed.
     * 
     * @param request contains the request parameters we need to have
     * @param response contains the client's url to write to
     * @throws IOException in case problems occurred accessing a properties file
     */
    public void doPost(HttpServletRequest request, HttpServletResponse response)
        throws IOException
    {
        //checks whether it's necessary to read/update the global settings
        try {
            this.readGlobalProperties();

            //Check whether the request contains url parameters: if not,
            //the simple default form is shown, else the current provider's individual one
            if (!request.getParameterNames().hasMoreElements()) {
                //in case no url parameters are defined, show the default form with using
                //the parameters of the system provider
                request.getSession().setAttribute(JSPBEAN,
                        new JSPBean(SYSTEMPROVIDER, globalProperties, mailFormProviders));
                response.sendRedirect(response.encodeRedirectURL(SITE_DEFAULTFORM));
            }
            else {
                String sProviderName = getParameterIgnoreCase(request, PARAM_PROVIDER);
                boolean bProviderExists = mailFormProviders.hasProvider(sProviderName);

                //show error message when somebody tried to use a non-existent provider name; if a
                //spam error url is defined, redirect the client to this url
                if (!bProviderExists) {
                    if (globalProperties.getSpamErrorURL() != null
                            && globalProperties.getSpamErrorURL().length() > 0) {
                        response.sendRedirect(
                        		response.encodeRedirectURL(globalProperties.getSpamErrorURL()));
                    }
                    else {
                        throw new MailWebFormException(
                                "MailWebForm was not called correctly! Required parameter wrong or missing.");
                    }
                }
                else {
                    //get the provider-specific settings (after eventual reload of the file)
                    mailFormProviders.checkProviderSettings(sProviderName);

                    String sProviderSuccessURL = mailFormProviders.getSuccessURL(
                    		sProviderName, globalProperties.getSuccessURL());

                    try {
                        //read the URL parameters and send the mail
                        getRequestParametersAndSendEmail(
                        		request, 
                        		sProviderName, 
                        		mailFormProviders.getSMTPHost(sProviderName, globalProperties.getSMTPHost()),
                                mailFormProviders.getSMTPPort(sProviderName, globalProperties.getSMTPPort()), 
                                mailFormProviders.getToName(sProviderName, ""), 
                                mailFormProviders.getToAddress(sProviderName, ""));

                        //find out whether the provider's success URL (that may not be defined and
                        //therefore equals the class' (default) SUCCESS_URL) equals "". If so, show 
                        //the own output, ...
                        if (sProviderSuccessURL.equals("")) {
                            this.printOutput("Message sent successfully!", response);
                        }
                        //...else redirect the client to the specific URL (same in case of errors!)
                        else {
                            response.sendRedirect(response.encodeRedirectURL(sProviderSuccessURL));
                        }
                    } catch (MessagingException me) {
                        handleSendingError(
                        		me, 
                        		mailFormProviders.getErrorURL(sProviderName, globalProperties.getErrorURL()), 
                        		response);
                    }
                }
            }
        } catch (MailWebFormException mfwe) {
            showErrorScreen(mfwe, request, response);
        }
    }

    /**
     * This is where things happen. The required parameters are read, their hierarchy is determined,
     * and the highest ranking settings are added to an instance of
     * org.alltimeflashdreamer.email.MailSender. The exceptions that may be thrown during forming
     * and sending of a mail are wrapped into javax.mail.MessagingException objects.
     * 
     * @param request where to read the url parameters from
     * @param sProviderName one of those defined in the global settings file
     * @param sSMTPHost the provider's own SMTP host setting (may be undefined)
     * @param iSMTPPort the provider's own SMTP port setting (may be undefined)
     * @param sRecipientName the provider's name setting (may be undefined)
     * @param sRecipientEmail the provider's address setting (may be undefined)
     * @throws MailWebFormException if the validation of the input parameters failed
     * @throws MessagingException thrown by the MailSender class, in case something with the address
     *             format or the delivering of the mail went wrong
     */
    private void getRequestParametersAndSendEmail(HttpServletRequest request, String sProviderName,
            String sSMTPHost, int iSMTPPort, String sRecipientName, String sRecipientEmail)
        throws MailWebFormException, MessagingException
    {
        // Get the URL parameters
        String sSenderName = "";
        String sSenderEmail = "";
        String sEmailSubject = "";
        String sEmailText = "";
        sRecipientName = (sRecipientName == null) ? "" : sRecipientName;

        TreeMap parameters = new TreeMap();

        // search request for each required key and put the rest into the list
        // of additional key/value pairs
        Enumeration eParameterNames = request.getParameterNames();

        while (eParameterNames.hasMoreElements()) {
            String sNextName = (String)eParameterNames.nextElement();

            //avoid already used parameters
            if (sNextName.equalsIgnoreCase(PARAM_PROVIDER)
                    || sNextName.equalsIgnoreCase(PARAM_CONFIGFILE)) {//for backward compatibility,
                                                                      // isn't used by this app
                                                                      // anymore

                continue;
            }

            //construct mail (subject may be undefined)
            if (sNextName.equalsIgnoreCase(PARAM_FROMNAME)) {
                sSenderName = request.getParameter(sNextName);
            }
            else if (sNextName.equalsIgnoreCase(PARAM_FROMEMAIL)) {
                sSenderEmail = request.getParameter(sNextName);
                if (StringUtils.isEmpty(sSenderEmail))
                    throw new MailWebFormException("The sender's email address must not be empty!");
            }
            else if (sNextName.equalsIgnoreCase(PARAM_TONAME)) {
                sRecipientName = request.getParameter(sNextName);
            }
            else if (sNextName.equalsIgnoreCase(PARAM_SUBJECT)) {
                sEmailSubject = request.getParameter(sNextName);
                if (sEmailSubject == null)
                    sEmailSubject = "";
            }
            else if (sNextName.equalsIgnoreCase(PARAM_TEXT)) {
                sEmailText = request.getParameter(sNextName);
            }
			// process all available html variables (except provider and config file)
			parameters.put(sNextName, request.getParameter(sNextName));
        }

        if (!StringUtils.isEmpty(mailFormProviders.getProvider(sProviderName).getTemplate())) {
            String sProviderDirectory = mailFormProviders
            		.getProvider(sProviderName).getProviderDirectory();

            if (StringUtils.isEmpty(sProviderDirectory)) {
                sProviderDirectory = getConfigDirProperty();
            }

            //parse the template and fill in the parameters, if a template is defined
            sEmailText = processTemplate(
            		sProviderDirectory
                    	+ mailFormProviders.getProvider(sProviderName).getTemplate(), 
                    parameters,
                    sEmailText);
        }
        else {
            //add the (possible) other parameter names and values to the beginning of the
            // EMAIL_TEXT,
            //each name/value pair on one separate line
            String sParameterText = buildParameterString(parameters);

            if (!sParameterText.equals("")) { //add text only if there are name/value pairs!
                sEmailText = sParameterText + 
                		"\n----------------------\n\n" + sEmailText;
            }
        }

        //Create the mail sender and send its mail
        MailSender mail = new MailSender(sSMTPHost, iSMTPPort, APPNAME);
        mail.setFrom(sSenderName, sSenderEmail);
        mail.setTo(sRecipientName, sRecipientEmail);
        mail.setSubject(sEmailSubject);
        mail.setText(sEmailText);
        mail.setHeader("X-originating-IP", request.getRemoteAddr());
        mail.sendMessage();
    }

    /**
     * Builds a list of "="-separated key/value pairs out of the two given lists of parameter names
     * and values.
     * 
     * @param parameters the list of not-application-specific parameters
     * @return the String containing the key/value pairs, each on a separate line
     */
    private String buildParameterString(TreeMap parameters)
    {
        StringBuffer sbParameterText = new StringBuffer();

        for (Iterator iterKeys = parameters.keySet().iterator(); iterKeys.hasNext();) {
            String sNextKey = (String)iterKeys.next();
            sbParameterText.append(sNextKey).append(" = ")
            		.append((String)parameters.get(sNextKey)).append("\n");
        }

        return sbParameterText.toString();
    }

    /**
     * Processes the template file specified by the given name. "Processing" means that the file is
     * searched line by line for placeholders that are then replaced with the relative parameter of
     * the given list. The only exception is the mail text itself since this is not a parameter but
     * a separate, application-specific value.
     * 
     * @param sTemplateFileName the fully qualified name of the template file
     * @param parameters the list of not-application-specific parameters
     * @param sEmailText the mail text (replaces the placeholder TEXT)
     * @return the fully processed template as a String
     * @throws MailWebFormException if the reading of the template file fails somehow
     */
    private String processTemplate(String sTemplateFileName, TreeMap parameters, String sEmailText)
        throws MailWebFormException
    {
        File template = new File(sTemplateFileName);
        String sErrorMessage = "Mail template \"" + sTemplateFileName
                + "\" doesn't exist or can't be read by the application!";

        try {
            if (!template.canRead())
                throw new MailWebFormException(sErrorMessage);
        } catch (SecurityException se) {
            throw new MailWebFormException(sErrorMessage, se);
        }

        //read the template line by line and search for the placeholder entries;
        //if one is found, replace it with the relative entry within the given
        //list of parameters (case-insensitive check)
        BufferedReader reader = null;
        String sLine = null;
        String sReturnValue = "";

        try {
            reader = new BufferedReader(new FileReader(template));

            while ((sLine = reader.readLine()) != null) {
                //replace each placeholder on this line...
                while (sLine.indexOf(PLACEHOLDER_START) >= 0
                        && sLine.indexOf(PLACEHOLDER_END) >= 0) {
                    sLine = replacePlaceHolder(sLine, parameters, sEmailText);
                }

                //...and add it to the return value
                sReturnValue += (sLine + "\n");
            }
        } catch (FileNotFoundException fnfe) {
            throw new MailWebFormException("Template file \"" + sTemplateFileName
                    + "\" doesn't exist!");
        } catch (IOException ioe) {
            throw new MailWebFormException("Error occurred while reading the template file \""
                    + sTemplateFileName + "\"!", ioe);
        } finally {
            if (reader != null) {
                try {
                    reader.close();
                } catch (Exception ignore) {}
            }
        }

        return sReturnValue;
    }

    /**
     * Replaces the first placeholder on the given line with the relative value of the given map. If
     * such a placeholder key isn't contained in the parameter list, it's replaced with an empty
     * string.
     * 
     * @param sLine the line to scan for placeholders
     * @param parameters the list of not-application-specific parameters
     * @param sEmailText the replacement for the placeholder named "text"
     * @return the processed line
     */
    private String replacePlaceHolder(String sLine, TreeMap parameters, String sEmailText)
    {
        String sPlaceHolderKey = sLine.substring(
        		sLine.indexOf(PLACEHOLDER_START) + PLACEHOLDER_START.length(), 
        		sLine.indexOf(PLACEHOLDER_END));
        String sLineBeginning = sLine.substring(0, sLine.indexOf(PLACEHOLDER_START));
        String sLineEnd = sLine.substring(
        		sLine.indexOf(PLACEHOLDER_END) + PLACEHOLDER_END.length());
        String sReplacement = "";

        if (sPlaceHolderKey.equalsIgnoreCase(PARAM_TEXT)) {
            sReplacement = sEmailText;
        }
        else {
            for (Iterator iterKeys = parameters.keySet().iterator(); iterKeys.hasNext();) {
                String sNextKey = (String)iterKeys.next();

                if (sNextKey.equalsIgnoreCase(sPlaceHolderKey)) {
                    sReplacement = (String)parameters.remove(sNextKey);
                    break;
                }
            }
        }

        return sLineBeginning.concat(sReplacement).concat(sLineEnd);
    }

    /**
     * Does what is recommended in case an error occurred while sending the mail (which is described
     * in the MessagingException parameter). This is either showing the own message (in case the
     * error url parameter is empty or null) or showing the given error site.
     * 
     * @param me error that occurred while sending the mail
     * @param sProviderErrorURL the error site to be shown (may be empty or null, which leads to the
     *            showing of the Java error message)
     * @param response the response object to send the output to
     * @throws MailWebFormException wrapped, error-describing MessagingException
     * @throws IOException thrown on redirecting the client
     */
    private void handleSendingError(MessagingException me, String sProviderErrorURL,
            HttpServletResponse response) throws MailWebFormException, IOException
    {
        if (sProviderErrorURL == null || sProviderErrorURL.equals("")) {
            throw new MailWebFormException(
            		me.getClass().getName() + ": " + me.getMessage(), 
            		me.getNextException());
        }

        response.sendRedirect(response.encodeRedirectURL(sProviderErrorURL));
    }
}