package org.alltimeflashdreamer.util;

import java.io.*;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;

import java.text.*;

import java.util.*;


/**
 * StringUtils<br> Description: This class provides some often used String
 * and/or String[] operations.
 * 
 * <p>
 * Organization: All Time Flash Dreamer
 * </p>
 *
 * @author Copyright (c) 2002 by <a href="mailto:florian@steinsiepe.ch">Florian
 * 		   Steinsiepe</a>
 * @version 1.0
 */
public final class StringUtils
{
	/**
	 * the list of strings for each multiplier of 1024 (bytes), such as "Bytes"
	 * or "KB" or "GB"
	 */
	public final static String[] MULTIPLIER_TEXTS = {
		"Bytes", "KB", "MB", "GB", "TB", "PB"
	};
	
	/** used as default values for the formatPeriod method */
	private final static String[] PERIOD_TEXTS = {"d", "h", "min", "sec"};

	/** a handy accessor for the file.separator system property */
	public final static String FILESEPARATOR = System.getProperty(
			"file.separator");
	private final static String FORWARDSLASH = "/";
	private final static int FILESEPARATORLENGTH = FILESEPARATOR.length();
	private static NumberFormat nf = NumberFormat.getInstance();

	static {
		nf.setMinimumFractionDigits(0);
		nf.setMaximumFractionDigits(3);
	}

	/** Used by the hash method. */
	private static MessageDigest digest = null;

	//---------------------------------- private methods ----------------------------------------//

	/**
	 * Must not be called but through the getInstance method
	 */
	private StringUtils() {}

	/**
	 * Static accessor for an instance of this class, to be used e.g. within an
	 * EJB container to avoid static class access from within beans.
	 *
	 * @return a new instance of this class
	 */
	public static StringUtils getInstance()
	{
		return new StringUtils();
	}

	/**
	 * Sorts the array using a primary, i.e. case insensitive
	 * java.text.Collator instance. This transforms the list parameter into an
	 * ascendingly sorted array. To know which String is where <i>after</i>
	 * the sorting, the returned list contains the original indexes (i.e.
	 * positions within the given list). This allows to loop over the indexes
	 * instead of using the sorted list (useful for more complex objects that
	 * need to be sorted by e.g. their name entries).<br>
	 * Note: This method uses the primitive sort algorithm and isn't
	 * performing for long lists.
	 *
	 * @param list the list of values to sort
	 *
	 * @return a list of indexes sorted in the same way as the given list
	 *
	 * @see Collator
	 * @see Collator#PRIMARY
	 */
	public static int[] caseInsensitiveSort(String[] list)
	{
		//consistency checks
		if ((list == null) || (list.length < 1)) {
			return ((list == null) ? null : new int[] {});
		}

		//build and fill the index array (which is sorted too and returned at the end)
		int[] iReturnValue = new int[list.length];

		for (int i = 0; i < iReturnValue.length; i++)
			iReturnValue[i] = i;

		//job start
		String sTemp;
		int iTemp;
		Collator collator = Collator.getInstance();
		collator.setStrength(Collator.PRIMARY);

		for (int i = 0; i < (list.length - 1); i++) {
			for (int j = (i + 1); j < list.length; j++) {
				int iCompare = collator.getCollationKey(list[i]).compareTo(collator.getCollationKey(
							list[j]));

				if (iCompare > 0) {
					sTemp = list[i];
					list[i] = list[j];
					list[j] = sTemp;
					iTemp = iReturnValue[i];
					iReturnValue[i] = iReturnValue[j];
					iReturnValue[j] = iTemp;
				}
			}
		}

		return iReturnValue;
	}

	/**
	 * Tokenizes sInput using a StringTokenizer, specified by sSeparator, and
	 * returns an ArrayList filled with the tokens.
	 *
	 * @param sInput a somehow separated string
	 * @param sSeparator the separator to use
	 *
	 * @return the list of separated values (substrings of sInput)
	 */
	public static ArrayList tokenizeToArrayList(String sInput, String sSeparator)
	{
		ArrayList al = new ArrayList();

		if ((sInput != null) && (sSeparator != null)) {
			StringTokenizer st = new StringTokenizer(sInput, sSeparator);

			while (st.hasMoreElements())
				al.add(st.nextElement());
		}

		return al;
	}

	/**
	 * Tokenizes sInput using a StringTokenizer, specified by sSeparator, and
	 * returns a String[] filled with the tokens (as strings).
	 *
	 * @param sInput a somehow separated string
	 * @param sSeparator the separator to use
	 *
	 * @return the list of separated values (substrings of sInput)
	 */
	public static String[] tokenizeToStringArray(String sInput,
		String sSeparator)
	{
		return (String[])tokenizeToArrayList(sInput, sSeparator).toArray(new String[] {
				
			});
	}

	/**
	 * Loads the stack trace of the Throwable into a StringWriter and returns
	 * the toString value of the writer.
	 *
	 * @param t the Throwable to get the stack trace of
	 *
	 * @return the stack trace of the method in a single String
	 */
	public static String getStackTrace(Throwable t)
	{
		StringWriter sw = new StringWriter();
		t.printStackTrace(new PrintWriter(sw));

		return sw.toString();
	}

	/**
	 * Returns whether the given test string is null or equals "".
	 *
	 * @param sTestString the String to test
	 *
	 * @return whether it's null or equals the empty String
	 */
	public static boolean isEmpty(String sTestString)
	{
		return ((sTestString == null) || sTestString.equals(""));
	}

	/**
	 * Formats the given size (as a long, specifying the total byte count) into
	 * an outputtable String, like "393 Bytes" or "3.567 MB".
	 *
	 * @param lSizeInBytes the byte amount / size to format (negative sizes
	 * 		  aren't formatted!)
	 *
	 * @return the fully formatted size string
	 */
	public static String formatSize(long lSizeInBytes)
	{
		double dSize = lSizeInBytes;
		int iCtr = 1;
		int iTimes1024 = 0;

		while (dSize >= 1024.0) {
			iTimes1024++;
			dSize /= 1024.0;
		}

		StringBuffer sb = new StringBuffer();

		switch (iTimes1024) {
			case 0:
				sb.append(lSizeInBytes).append(" ").append(MULTIPLIER_TEXTS[0])
					.toString();

				break;

			case 1:
				sb.append(nf.format(dSize)).append(" ")
					.append(MULTIPLIER_TEXTS[1]).toString();

				break;

			case 2:
				sb.append(nf.format(dSize)).append(" ")
					.append(MULTIPLIER_TEXTS[2]).toString();

				break;

			case 3:
				sb.append(nf.format(dSize)).append(" ")
					.append(MULTIPLIER_TEXTS[3]).toString();

				break;

			case 4:
				sb.append(nf.format(dSize)).append(" ")
					.append(MULTIPLIER_TEXTS[4]).toString();

				break;

			case 5:default:
				sb.append(nf.format(dSize)).append(" ")
					.append(MULTIPLIER_TEXTS[5]).toString();

				break;
		}

		return sb.toString();
	}

	/**
	 * Formats the given amount of milliseconds into something like "2d 1h 
	 * 23min 37.123sec", indicating a period, not an absolute time, like 
	 * it's used in GUI's that count the passed time since e.g. the download 
	 * started.
	 *
	 * @param lMilliseconds how long is this period?
	 *
	 * @return something like described above (at most hours, no days or months
	 * 		   or so, and no milliseconds too!)
	 */
	public static String formatPeriod(long lMilliseconds)
	{
		return formatPeriod(lMilliseconds, PERIOD_TEXTS);
	}
	
	/**
	 * Same as formatPeriod(long), but uses the given String[4] to format
	 * the days, hours, minutes and seconds. If the text list is null or
	 * not of length 4, the default texts (english abbreviations) are used.
	 * 
	 * @param lMilliseconds how long is this period?
	 * @param sTexts	the texts to be used as descriptors
	 * 
	 * @return something like "2d 1h 23min 37.123sec", but using the given
	 * texts
	 */
	public static String formatPeriod(long lMilliseconds, String[] sTexts)
	{
		if (sTexts == null || sTexts.length != PERIOD_TEXTS.length)
			sTexts = PERIOD_TEXTS;
		
		StringBuffer sb = new StringBuffer();

		if (lMilliseconds > 86400000L) {
			long lDays = lMilliseconds / 86400000L;
			lMilliseconds -= (lDays * 86400000L);
			
			if (lDays > 0) {
				sb.append(lDays).append(sTexts[0].trim()).append(' ');
			}
		}
		
		if (lMilliseconds > 3600000L) {
			long lHours = lMilliseconds / 3600000L;
			lMilliseconds -= (lHours * 3600000L);

			if (lHours > 0) {
				sb.append(lHours).append(sTexts[1].trim()).append(' ');
			}
		}

		if (lMilliseconds > 60000L) {
			long lMinutes = lMilliseconds / 60000L;
			lMilliseconds -= (lMinutes * 60000L);

			if (lMinutes > 0) {
				sb.append(lMinutes).append(sTexts[2].trim()).append(' ');
			}
		}

		long lSeconds = lMilliseconds / 1000L;
		return sb.append(lSeconds).append('.').append(lMilliseconds % 1000)
			.append(sTexts[3].trim()).toString();
	}

	/**
	 * If we're running on a machine having "\" as the System's file.separator
	 * property (such as Windows), the backslashes need to be replaceed with
	 * "/" (forward slashes), otherwise the next character will be escaped
	 * within the sequence which leads to inpredictable/wrong behaviour.
	 *
	 * @param sPath the (directory/path) string to escape
	 *
	 * @return the escaped string, if already "/", else sPath itself
	 */
	public static String replaceBackslashes(String sPath)
	{
		if (sPath == null) {
			return sPath;
		}

		if (FILESEPARATOR.equals(FORWARDSLASH)) {
			return sPath;
		}

		if (FILESEPARATORLENGTH == 1) {
			return sPath.replace('\\', '/');
		}

		StringTokenizer st = new StringTokenizer(sPath, FILESEPARATOR);
		StringBuffer sbEscaped = new StringBuffer();

		while (st.hasMoreTokens())
			sbEscaped.append(st.nextToken()).append(FORWARDSLASH);

		//return buffer's string (minus the last to FILESEPARATOR instances, if sPath
		//itself didn't end with a FILESEPARATOR)
		return ((sPath.endsWith(FILESEPARATOR)) ? sbEscaped.toString()
												: sbEscaped.substring(0,
			sbEscaped.length() - FORWARDSLASH.length()));
	}

	/**
	 * Appends either a forward slash or a file separator to the given
	 * directory string, depending on the boolean parameter, but <i>only if it
	 * doesn't end with neither a forward slash nor a file separator</i> . In
	 * case of non-Windows operating systems, the appended string is a forward
	 * slash anyway.
	 *
	 * @param sDirectory the directory string to modify, if necessary; if
	 * 		  sDirectory is null, it's treated like an empty string
	 * @param bUseForwardSlash whether to add a forward slash or a file
	 * 		  separator
	 *
	 * @return the eventually modified string
	 */
	public static String appendFileSeparator(String sDirectory,
		boolean bUseForwardSlash)
	{
		if (isEmpty(sDirectory)) {
			sDirectory = "";
		}

		if (!sDirectory.endsWith("/") && !sDirectory.endsWith("\\")) {
			return sDirectory += ((bUseForwardSlash) ? "/" : FILESEPARATOR);
		}

		return sDirectory;
	}

	/**
	 * Returns the given string itself, if the isEmpty method returns true,
	 * else the trimmed input string is returned.
	 *
	 * @param sInput the string to trim, if not empty
	 *
	 * @return the empty or the trimmed string
	 */
	public static String getTrimmedString(String sInput)
	{
		if (isEmpty(sInput)) {
			return sInput;
		}

		return sInput.trim();
	}

	/**
	 * If the given input is below 10, it's transformed into the form "0" +
	 * iInput, else iInput is returned as a String only.
	 *
	 * @param iInput the value to be transformed
	 *
	 * @return String    a String representation of iInput being at least to
	 * 		   digits long
	 */
	public static String getTwoDigitInt(int iInput)
	{
		if (iInput < 10) {
			return "0" + iInput;
		}

		return "" + iInput;
	}

	/**
	 * Removes each newline sign ( "\n" ) from the given string and returns the
	 * updated one.
	 *
	 * @param sInput the string to remove the newline signs from
	 *
	 * @return the newline-free string
	 */
	public static String removeNewlines(String sInput)
	{
		return removeCharacters(sInput, '\n');
	}

	/**
	 * Removes each instance of the given character from the given string and
	 * returns  the updated one.
	 *
	 * @param sInput the string to remove each instance of the given character
	 * 		  from
	 * @param character the character to search for
	 *
	 * @return the updated string
	 */
	public static String removeCharacters(String sInput, char character)
	{
		StringTokenizer st = new StringTokenizer(sInput, "" + character);
		StringBuffer sb = new StringBuffer();

		while (st.hasMoreTokens())
			sb.append(st.nextToken());

		return sb.toString();
	}

	/**
	 * Capitalizes the given string and returns the modified version.
	 * "Capitalize" means that the first character is transformed into the
	 * upper case version, while each following one is transformed to the
	 * lower case.
	 *
	 * @param sInput the string which is to be capitalized
	 *
	 * @return the modified string
	 */
	public static String capitalize(String sInput)
	{
		if (isEmpty(sInput)) {
			return sInput;
		}

		StringBuffer sb = new StringBuffer();

		for (int i = 0; i < sInput.toCharArray().length; i++) {
			if (i == 0) {
				sb.append(Character.toUpperCase(sInput.charAt(i)));
			} else {
				sb.append(Character.toLowerCase(sInput.charAt(i)));
			}
		}

		return sb.toString();
	}

	/**
	 * Converts the given String[] <i>containing parsable long values</i> into
	 * the corresponding long[]. If the given String[] is null or empty, a
	 * long[0] is returned; if one of the values of the given list isn't
	 * parsable into a long, an exception is thrown.
	 *
	 * @param sValues the parsable list of Strings
	 *
	 * @return the parsed list, transformed into a long[]
	 *
	 * @throws NumberFormatException if one of the given strings can't be
	 * 		   parsed into a long
	 */
	public static long[] toLongArray(String[] sValues)
		throws NumberFormatException
	{
		if ((sValues == null) || (sValues.length == 0)) {
			return new long[0];
		}

		long[] lReturnValue = new long[sValues.length];

		for (int i = 0; i < lReturnValue.length; i++)
			lReturnValue[i] = new Long(sValues[i]).longValue();

		return lReturnValue;
	}

	/**
	 * Converts the given String[] <i>containing parsable int values</i> into
	 * the corresponding int[]. If the given String[] is null or empty, an
	 * int[0] is returned; if one of the values of the given list isn't
	 * parsable into a int, an exception is thrown.
	 *
	 * @param sValues the parsable list of Strings
	 *
	 * @return the parsed list, transformed into a int[]
	 *
	 * @throws NumberFormatException if one of the given strings can't be
	 * 		   parsed into a long
	 */
	public static int[] toIntArray(String[] sValues)
		throws NumberFormatException
	{
		if ((sValues == null) || (sValues.length == 0)) {
			return new int[0];
		}

		int[] iReturnValue = new int[sValues.length];

		for (int i = 0; i < iReturnValue.length; i++)
			iReturnValue[i] = new Integer(sValues[i]).intValue();

		return iReturnValue;
	}

	/**
	 * Turns an array of bytes into a String representing each byte as an
	 * unsigned hex number.
	 * 
	 * <p>
	 * Method by Santeri Paavolainen, Helsinki Finland 1996<br>
	 * (c) Santeri Paavolainen, Helsinki Finland 1996<br>
	 * Distributed under LGPL.
	 * </p>
	 *
	 * @param hash an array of bytes to convert to a hex-string
	 *
	 * @return generated hex string
	 */
	public static final String toHex(byte[] hash)
	{
		StringBuffer buf = new StringBuffer(hash.length * 2);
		int i;

		for (i = 0; i < hash.length; i++) {
			if ((hash[i] & 0xff) < 0x10) {
				buf.append("0");
			}

			buf.append(Long.toString(hash[i] & 0xff, 16));
		}

		return buf.toString();
	}

	/**
	 * Hashes a String using the Md5 algorithm and returns the result as a
	 * String of hexadecimal numbers. This method uses synchronized access to
	 * a singleton MessageDigest instance to avoid excessive MessageDigest
	 * object  creation. If calling this method becomes a bottleneck in your
	 * code, you may  wish to maintain a pool of MessageDigest objects instead
	 * of using this method.
	 * 
	 * <p>
	 * A hash is a one-way function -- that is, given an input, an output is
	 * easily computed. However, given the output, the input is almost
	 * impossible to compute. This is useful for passwords since we can store
	 * the hash and a hacker will then have a very hard time determining the
	 * original password.
	 * </p>
	 *
	 * @param data the String to compute the MD5 hash of
	 *
	 * @return a hashed version of the passed-in String, transformed into a hex
	 * 		   string
	 *
	 * @see #toHex(byte[])
	 */
	public static final String hash(String data)
	{
		if (digest == null) {
			try {
				digest = MessageDigest.getInstance("MD5");
			} catch (NoSuchAlgorithmException nsae) {
				System.err.println("Failed to load the MD5 MessageDigest!");
				nsae.printStackTrace();
			}
		}

		//compute the hash and transform it into a hex string
		synchronized (digest) {
			digest.update(data.getBytes());

			return toHex(digest.digest());
		}
	}
	
	/**
	 * Replaces placeholders like "${0}" with the values at the specified index
	 * in the given array and returns the modified string.
	 * 
	 * @param sInput	the string to be modified
	 * @param sValues	the values to replace the placeholders with
	 * @return	the modified string
	 */
	public static String replacePlaceholders(String sInput, String[] sValues) {
	    StringBuffer sb = new StringBuffer();
	    
	    while (sInput.indexOf("${") != -1) {
	        sb.append(sInput.substring(0, sInput.indexOf("${")));
	        sInput = sInput.substring(sInput.indexOf("${") + 2);
	        int iIndex = new Integer(sInput.substring(0, sInput.indexOf("}"))).intValue();
	        sb.append(sValues[iIndex]);
	        sInput = sInput.substring(sInput.indexOf("}") + 1);
	    }
	    return sb.append(sInput).toString();
	}
}
