package org.alltimeflashdreamer.util.email;

import java.io.UnsupportedEncodingException;

import java.util.*;

import javax.mail.*;
import javax.mail.internet.*;


/**
 * Title: MailSender<br>
 * Description: Constructs a mail (a mime message) and delivers it using the
 * javax.mail package (final release, as provided in the J2EE 1.3 distribution).<br>
 * General: Mail addresses are written either in the RFC822 standard or as
 * simple addresses, even without a domain part. This means that in case a
 * mail has several recipients, their addresses are appended to the address,
 * separated by a comma. Any other content is mime-decoded, which means that
 * any characters and any attachments are supported.
 *
 * @author Copyright (c) 2001 - 2004 by <a
 * 		   href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
 * @version 1.1
 */
public class MailSender
{
	private MimeMessage message;
	private String sCharset = "iso8859-1";

	/**
	 * The constructor receives a Properties object filled with mail-related
	 * key-value pairs.<br>
	 * The following values may be defined:
	 * 
	 * <ul>
	 * <li>
	 * mail.transport.protocol<br> default is "smtp"; if overwritten, the
	 * mail.&lt;protocol_name&gt;.provider value must be defined too,
	 * containing the class name of the implementation of the new protocol
	 * </li>
	 * <li>
	 * mail.smtp.host<br> default value is the name of the local machine
	 * </li>
	 * <li>
	 * mail.smtp.user=&lt;username&gt;<br> default value is the current user
	 * name, retrieved from the System's "user.name" property
	 * </li>
	 * </ul>
	 * 
	 *
	 * @param sessionProperties Mail server configuration bundle, must not be
	 * 		  null
	 * @param sXMailer Optional parameter (i.e. may be null) that is added to
	 * 		  the mail header, within the "X-Mailer" attribute
	 *
	 * @throws MessagingException Java Mail API's own exception, is to be
	 * 		   handled by the caller
	 */
	public MailSender(Properties sessionProperties, String sXMailer)
		throws MessagingException
	{
		init(sessionProperties, sXMailer);
	}
	
	/**
	 * Calls the other constructor with a Properties instance filled with
	 * the given values.
	 * 
	 * @param sSMTPHost	the host to send the mail to
	 * @param iSMTPPort	the port of the SMTP server at the specified host
	 * @param sXMailer Optional parameter (i.e. may be null) that is added to
	 * 		  the mail header, within the "X-Mailer" attribute
	 *
	 * @throws MessagingException Java Mail API's own exception, is to be
	 * 		   handled by the caller
	 */
	public MailSender(String sSMTPHost, int iSMTPPort, String sXMailer)
		throws MessagingException
	{
		Properties sessionProperties = new Properties();
		sessionProperties.put("mail.smtp.host", sSMTPHost);
		sessionProperties.put("mail.smtp.port", new Integer(iSMTPPort).toString());

		init(sessionProperties, sXMailer);
	}
	
	/**
	 * Initializes the mail session and sets the default header attributes.
	 * 
	 * @param sessionProperties Mail server configuration bundle, must not be
	 * 		  null
	 * @param sXMailer Optional parameter (i.e. may be null) that is added to
	 * 		  the mail header, within the "X-Mailer" attribute
	 *
	 * @throws MessagingException Java Mail API's own exception, is to be
	 * 		   handled by the caller
	 */
	private void init(Properties sessionProperties, String sXMailer)
		throws MessagingException
	{
		//build session and message
		Session session = Session.getInstance(sessionProperties);
		message = new MimeMessage(session);

		//set special header stuff
		setHeader("X-Mailer",
			((sXMailer == null) ? getClass().getName() : sXMailer));

		String sVersion = session.getProvider("smtp").getVersion();
		setHeader("X-Mailer-Engine_Base_Language",
			"Java Mail API" + ((sVersion == null) ? "" : (" " + sVersion)));
	}

	/**
	 * Allows to set some key/value pair to the header. If this pair isn't
	 * formatted according to the specification in the RFC 822, an exception
	 * is thrown.
	 * 
	 * @param sKey	the header key, typically starting with a capital X
	 * @param sValue	the header value
	 * @throws MessagingException	if the header isn't applyable to the message
	 * according to th specification
	 * @see MimeMessage#setHeader(String, String)
	 */
	public void setHeader(String sKey, String sValue)
		throws MessagingException
	{
		message.setHeader(sKey, sValue);
	}

	/**
	 * Sets a new character set, if wished (default is ISO8859-1). This must be
	 * called before setting address or content values, because the character
	 * setting is used for those procedures.
	 *
	 * @param sCharacterSet if not defined, the default is used
	 */
	public void setCharacterSet(String sCharacterSet)
	{
		this.sCharset = sCharacterSet;
	}

	/**
	 * Sets/adds a from-address to the mail. The name may be null which means
	 * that no name is visible in the from line, only the address. If the name
	 * is not null, it's set using the current character set. The address will
	 * be used as the reply-to address too.
	 *
	 * @param sFromName Sender's name
	 * @param sFromAddress Sender's address (is used as the reply-to address
	 * 		  too)
	 *
	 * @throws MessagingException Wrapper exception, in case an
	 * 		   UnsupportedEncodingException or an AddressException was thrown
	 * 		   during setting of the addresses
	 */
	public void setFrom(String sFromName, String sFromAddress)
		throws MessagingException
	{
		try {
			message.setFrom(buildAddress(sFromName, sFromAddress));
			message.setReplyTo(message.getFrom());
		} catch (UnsupportedEncodingException uee) {
			throw new MessagingException("Unsupported character encoding in address!",
				uee);
		} catch (AddressException ae) {
			throw new MessagingException("Address is not formatted correctly!",
				ae);
		}
	}

	/**
	 * Sets/adds a to-address to the mail. The name may be null which means
	 * that no name is visible in the "To:" line, only the address. If the
	 * name is not null, it's set using the current character set. The address
	 * will be used as the reply-to address too.
	 *
	 * @param sToName Recipient's name
	 * @param sToAddress Recipient's address
	 *
	 * @throws MessagingException Wrapper exception, in case an
	 * 		   UnsupportedEncodingException or an AddressException was thrown
	 * 		   during setting of the addresses
	 */
	public void setTo(String sToName, String sToAddress)
		throws MessagingException
	{
		try {
			message.setRecipient(Message.RecipientType.TO,
				buildAddress(sToName, sToAddress));
		} catch (UnsupportedEncodingException uee) {
			throw new MessagingException("Unsupported character encoding in address!",
				uee);
		} catch (AddressException ae) {
			throw new MessagingException("Address is not formatted correctly!",
				ae);
		}
	}

	/**
	 * Sets/adds a cc-address to the mail. The name may be null which means
	 * that no name is visible in the "To:" line, only the address. If the
	 * name is not null, it's set using the current character set. The address
	 * will be used as the reply-to address too.
	 *
	 * @param sCcName cc's name
	 * @param sCcAddress cc's address
	 *
	 * @throws MessagingException Wrapper exception, in case an
	 * 		   UnsupportedEncodingException or an AddressException was thrown
	 * 		   during setting of the addresses
	 */
	public void setCc(String sCcName, String sCcAddress)
		throws MessagingException
	{
		try {
			message.setRecipient(Message.RecipientType.CC,
				buildAddress(sCcName, sCcAddress));
		} catch (UnsupportedEncodingException uee) {
			throw new MessagingException("Unsupported character encoding in address!",
				uee);
		} catch (AddressException ae) {
			throw new MessagingException("Address is not formatted correctly!",
				ae);
		}
	}

	/**
	 * Sets/adds a bcc-address to the mail. The name may be null which means
	 * that no name is visible in the "To:" line, only the address. If the
	 * name is not null, it's set using the current character set. The address
	 * will be used as the reply-to address too.
	 *
	 * @param sBccName bcc's name
	 * @param sBccAddress bcc's address
	 *
	 * @throws MessagingException Wrapper exception, in case an
	 * 		   UnsupportedEncodingException or an AddressException was thrown
	 * 		   during setting of the addresses
	 */
	public void setBcc(String sBccName, String sBccAddress)
		throws MessagingException
	{
		try {
			message.setRecipient(Message.RecipientType.BCC,
				buildAddress(sBccName, sBccAddress));
		} catch (UnsupportedEncodingException uee) {
			throw new MessagingException("Unsupported character encoding in address!",
				uee);
		} catch (AddressException ae) {
			throw new MessagingException("Address is not formatted correctly!",
				ae);
		}
	}

	/**
	 * Sets the subject of the message using the current character set.
	 *
	 * @param sSubject Message title
	 *
	 * @throws MessagingException thrown in case the title violates the RFC822
	 * 		   specification
	 */
	public void setSubject(String sSubject) throws MessagingException
	{
		message.setSubject(sSubject, sCharset);
	}

	/**
	 * Sets the text of the message using the current character set.
	 *
	 * @param sText Message text
	 *
	 * @throws MessagingException thrown in case the text violates the RFC822
	 * 		   specification
	 */
	public void setText(String sText) throws MessagingException
	{
		message.setText(sText, sCharset);
	}

	/**
	 * Sets the sending date to "now" and delivers the message.
	 *
	 * @throws MessagingException thrown in case something went wrong while
	 * 		   delivering the mail to the send mail server
	 */
	public void sendMessage() throws MessagingException
	{
		message.setSentDate(new Date());
		Transport.send(message);
	}

	/**
	 * Builds an address, depending on whether sName is null or not.
	 *
	 * @param sName Name for the address
	 * @param sAddress Address itself (US-ASCII characters only)
	 *
	 * @return InternetAddress guarantees a well-formatted content
	 *
	 * @throws AddressException thrown in case the address format is illegal
	 * @throws UnsupportedEncodingException thrown in case the characters are
	 * 		   not encodeable
	 */
	private InternetAddress buildAddress(String sName, String sAddress)
		throws AddressException, UnsupportedEncodingException
	{
		InternetAddress address = new InternetAddress(sAddress);

		if (sName != null) {
			address.setPersonal(sName, sCharset);
		}

		return address;
	}
}
